/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayInputStream;
import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.xml.bind.annotation.adapters.HexBinaryAdapter;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

import com.vaadin.data.Item;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.ui.Table;

/**
 * Provides access to a DBMS, actually MySQL. A DatabaseStorage maintains
 * a connection with the DBMS allowing different operations. Once the method
 * <code>close()<code> is executed the connection is closed.
 * 
 * Also provides property names as constants and hash generator for password
 * strings.
 * 
 * @author vcarceler <vcarceler@iespuigcastellar.xeill.net>
 * @author fjunyent <fjunyent@iespuigcastellar.xeill.net>
 * @author dperez <dperez@iespuigcastellar.xeill.net>
 */
public class DatabaseStorage {
	
	private ResourceBundle locale = ResourceBundle.getBundle("AttendanceManagerLocalization", new Locale( ResourceBundle.getBundle("AttendanceManager").getString("locale") ));

	private Connection con;
	private Statement statement;
	private String database;		// We need to do database+".group" in some queries
	//private AttendancemanagerApplication app;
	
	public static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
	public static SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm");
	
	private MessageDigest md;
	private HexBinaryAdapter hbinary = new HexBinaryAdapter();
	
	// Define property ID strings to refer table columns
	public static final String IDGROUP_PROPERTY = "idgroup";
	public static final String NAME_PROPERTY = "name";
	public static final String COMMENT_PROPERTY = "comment";
	public static final String STUDENTS_PROPERTY = "students";	// Used by GroupsQueryContainer
	public static final String IDSTUDENT_PROPERTY = "idstudent";
	public static final String IDTEACHER_PROPERTY ="idteacher";
	public static final String SURNAME1_PROPERTY = "surname1";
	public static final String SURNAME2_PROPERTY = "surname2";
	public static final String EMAIL_PROPERTY = "email";
	public static final String SMSPHONE_PROPERTY = "smsphone";
	public static final String LOGIN_PROPERTY = "login";
	public static final String FULLNAME_PROPERTY = "fullname";
	public static final String ADMIN_PROPERTY = "admin";
	public static final String PASSWORD_PROPERTY="password";
	public static final String START_PROPERTY="start";
	public static final String END_PROPERTY="end";
	public static final String TEACHER_IDTEACHER_PROPERTY="teacher_idteacher";
	public static final String MANAGERTEACHER_PROPERTY="tutor";
	public static final String IDCLASSBLOCK_PROPERTY="idclassblock";
	public static final String CLASSBLOCK_IDCLASSBLOCK_PROPERTY="classblock_idclassblock";
	public static final String SUBJECT_IDSUBJECT_PROPERTY="subject_idsubject";
	public static final String GROUP_IDGROUP_PROPERTY="group_idgroup";
	public static final String TERM_IDTERM_PROPERTY="term_idterm";
	public static final String TEACHERNAME_PROPERTY="teachername";
	public static final String TERMNAME_PROPERTY="termname";
	public static final String GROUPNAME_PROPERTY="groupname";
	public static final String SUBJECTNAME_PROPERTY="subjectname";
	public static final String DAYOFWEEK_PROPERTY="day_of_week";
	public static final String DAYOFWEEKSTRING_PROPERTY="dayofweekstring";
	public static final String DATE_PROPERTY="date";
	public static final String TIME_PROPERTY="time";
	public static final String STUDENT_IDSTUDENT_PROPERTY="student_idstudent";
	public static final String MISS_COUNT_PROPERTY="misscount";
	public static final String EXCUSEDMISS_COUNT_PROPERTY="excusedcount";
	public static final String DELAY_COUNT_PROPERTY="delaycount";
	public static final String EXPULSION_COUNT_PROPERTY="expulsioncount";
	public static final String IMAGE_ALERT_EMAILSEND_PROPERTY="alert-image-email-send";
	public static final String IMAGE_ALERT_SMSSEND_PROPERTY="alert-image-sms-send";
	public static final String MESSAGE_PREVIEW_PROPERTY= "message";
	
	public DatabaseStorage() {
		try {
			
			ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
			database = rb.getString("mysql_database");
			
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://" + rb.getString("mysql_server") + ":" + rb.getString("mysql_port") + "/" + database + "?autoReconnect=true";
			con = DriverManager.getConnection(url, rb.getString("mysql_user"), rb.getString("mysql_password"));
			statement = con.createStatement();
			Logger.log("DatabaseStorage() DB connection established");
			
			md = MessageDigest.getInstance("SHA-1"); 
			
		} catch(Exception e) {
			Logger.log("DatabaseStorage.DatabaseStorage()");
			System.err.println(e);
		}

	}
	
	/** 
	 * Closes the connection with database. Once closed the connection
	 * can't be reestablished, make a new instance if you want another
	 * connection.
	 */
	public void close() {
		try {
			con.close();
			Logger.log("DatabaseStorage.close() DB connection closed");
		} catch (SQLException e) {
			System.err.println("DatabaseStorage.close()");
			System.err.println(e);
			e.printStackTrace();
		}
	}

	/** Looks for the login/password pair in Teacher table, returns Teacher object if
	 * it's a real user or null if login fails.
	 * 
	 * @param login Login string to look for
	 * @param password Password string to look for
	 * @return Teacher corresponding to login/password pair or null 
	 */
	public Teacher loginUser(String login, String password) {
		
		Teacher user = null;
		
		try {
			
			String passwordhash = hbinary.marshal(md.digest(password.getBytes())).toLowerCase();
			
			ResultSet result = statement.executeQuery("SELECT idteacher, fullname, email, comment, admin FROM teacher WHERE teacher.login = '" + login + "' AND teacher.password = '" + passwordhash + "';");
			if(result.next()) {
				user = new Teacher(result.getInt("idteacher"), login, passwordhash, result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return user;
	}
	
	/** 
	 *	Query DBMS for login string existence
	 *
	 * @return true if login string exists false otherwise
	 */
	// QUÉ ES ESTO, QUIÉN LO UTILIZA ???
	public boolean loginExists(String login) {
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT login FROM teacher WHERE teacher.login = '" + login + "';");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}
	
	/** Returns true if given 'login' exists and is different that edited teacher , false otherwise */
	// QUÉ ES ESTO, QUIÉN LO UTILIZA ???
	public boolean loginExists(String login,Integer idTeacher) {
		boolean exists = false;
		
		try {
			//System.out.println(idTeacher);
			ResultSet result = statement.executeQuery("SELECT login FROM teacher WHERE teacher.login = '" + login + "' AND idteacher <> "+ idTeacher + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}

	/**
	 * Returns a collection with all the students of a group.
	 * 
	 * @param idgroup group to get students from
	 * @return Collection with all the students of the group, may be empty
	 */
	public Collection<Student> studentsOfGroup(int idgroup) {
		Vector<Student> vector = new Vector<Student>();
		
//añadido photoid
		try {
			ResultSet result = statement.executeQuery("SELECT idstudent, surname1, surname2, name, email, smsphone, photoid FROM student, group_has_student WHERE group_has_student.group_idgroup = " + idgroup + " AND group_has_student.student_idstudent = student.idstudent ORDER BY surname1, surname2, name;");
			while(result.next()) {
				vector.add(new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"), result.getString("name"), result.getString("email"),result.getString("smsphone"),result.getBytes("photoid")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
     * Adds a Group to a DDBB. A Group gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param group with ID 0
     * @return new ID or 0 if not added
	 */
	public int addGroup(Group group) {
		int id = 0;
		Integer idteacher=null;
		
		if (group.getID()!=0) {
			Logger.log("DatabaseStorage.addGroup() try to add group with no 0 ID\n" + group);
			return 0;
		}
		
		try {

			if (group.getIDTeacher()!=0) {idteacher=group.getIDTeacher();}
			statement.execute("INSERT INTO " + database + ".group VALUES('0', '" + group.getName() + "', '" + group.getComment() + "'," + idteacher + ");");

			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			group.setID(id);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	
	/**
	 * Añade al grupoA los alumnos del grupoB evitando duplicados. Es decir,
	 * los alumnos que forman parte de grupoB y de grupoA NO se añaden.
	 * 
	 * @param groupA ID del grupo al que se añadirán los alumnos
	 * @param groupB ID del grupo con los alumnos que se añadirán
	 * @return Número de registros añadidos
	 */
	public int addGroupToGroup(int groupA, int groupB) {
		int registros = 0;
		
		try {
			statement.execute("INSERT INTO group_has_student SELECT " + groupA + ", student_idstudent FROM group_has_student WHERE group_has_student.group_idgroup=" + groupB + " AND student_idstudent<>ALL (SELECT student_idstudent FROM group_has_student WHERE group_has_student.group_idgroup=" + groupA + ");");
			registros = statement.getUpdateCount();
		}catch(Exception e) {
			e.printStackTrace();
		}
		
		return registros;
	}
	
    /**
     * Updates group in DBMS.
     * 
     * @param group to update in DBMS
     * @return number of updated records
     */
    public int updateGroup(Group group) {
    	Integer id=group.getID();
    	String name=group.getName();
    	String comment=group.getComment();

    	Integer idteacher=null;
    	if (group.getIDTeacher()!=0){idteacher=group.getIDTeacher();}
        String sql="UPDATE " + database + ".group SET name='" + name + "' , comment='" + comment + "', teacher_idteacher=" + idteacher + " WHERE idgroup=" + id + ";";

        int res=0;
        
        try{
        	res = statement.executeUpdate(sql);
        } catch (SQLException e) {
        	e.printStackTrace();
        }

        return res;
    }
    
    /**
     * Checks if a group has students or is empty.
     * 
     * @param g Group to check if has any student
     * @return true if the group is not empty
     */
    public boolean groupHasStudents(Group g){
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT group_idgroup FROM group_has_student WHERE  group_idgroup=" + g.getID() + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;

    }
    
    /**
     * Returns true if teacher is tutor of one or more groups.
     * 
     * @param teacher
     * @return true in case teacher is tutor
     */
	public boolean isTutor(Teacher teacher) {
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM " + database + ".group WHERE " + database + ".group.teacher_idteacher=" + teacher.getID() +";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}
    
    /**
     * Check if a group has associated classblocks.
     * 
     * @param g Group to check if has any classblock.
     * @return true if the group has any classblock.
     */
    public boolean groupHasClassblocks(Group g){
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT idclassblock FROM classblock WHERE  group_idgroup=" + g.getID() + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
    }
    
    /**
     * Deletes a group from the DBMS
     * @param g group to delete
     */
    public void deleteGroup(Group g){
  
    	String sql="DELETE FROM " + database + ".group WHERE (idgroup='" + g.getID() +"');";
        try{
        	statement.execute(sql);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    	
    }
	
	
	/**
	 * Adds a Miss to a DDBB. A Miss gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param miss with ID 0
     * @return new ID or 0 if not added
	 */
	public int addMiss(Miss miss) {
		int id = 0;
		
		if (miss.getID()!=0) {
			Logger.log("DatabaseStorage.addMiss() try to add miss with no 0 ID\n" + miss);
			return 0;
		}
		
		try {
			
			statement.execute("INSERT INTO miss VALUES('0', '" + miss.getType() + "', '" + dateFormat.format(miss.getDate()) + "', '" + miss.getComment() + "', '" + miss.getIDStudent() + "', '" + miss.getIDTeacher() + "', '" + miss.getIDSubject() + "', '" + miss.getIDClassblock() + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			miss.setID(id);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	
	/** Update MISS TO MISS EXCUSED MISS
	 * A miss gets one ID when stored in database for the first time.*/
	public void updateExcusedtoMiss(Set<?> idmiss){
		String ids=null;
		for (Object id:idmiss){
			if (ids==null){
				ids=String.valueOf(id)+ ",";
			}else{
				ids=ids + String.valueOf(id) + ",";
			}
		}
		ids=ids.substring(0, ids.length()-1);
		try {
			statement.execute("UPDATE miss SET type=" + Miss.MISS + " WHERE idmiss IN (" + ids + ");");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	/** Update EXCUSED MISS TO MISS 
	 * A miss gets one ID when stored in database for the first time.*/
	public void updateMisstoExcused(Set<?> idmiss){
		String ids=null;
		for (Object id:idmiss){
			if (ids==null){
				ids=String.valueOf(id)+ ",";
			}else{
				ids=ids + String.valueOf(id) + ",";
			}
		}
		ids=ids.substring(0, ids.length()-1);
		try {
			statement.execute("UPDATE miss SET type=" + Miss.EXCUSED_MISS+ " WHERE idmiss IN (" + ids + ");");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns a QueryContainer with miss type;
	 * @param type Type of the miss to look for: Miss.MISS, Miss.EXPULSION, Miss.DELAY, Miss.EXCUSSED_MISS
	 * 
	 * @deprecated  Funciona mal. Su funcionalidad se ha dividido en los métodos
	 *              getMissQueryContainerBy*() y getMissQueryContainerAdminMode().
	 *			    {@link #getMissQueryContainerByStudent()}
	 *			    {@link #getMissQueryContainerByGroup()}
	 *			    {@link #getMissQueryContainerByTeacher()}
	 *			    {@link #getMissQueryContainerAdminMode()}
	 */
	@Deprecated // DIVIDIDO EN LOS MÉTODOS getMissQueryContainerBy*() Y get MissQueryContainerAdminMode()
	public QueryContainer getMissQueryContainer(int type, int idstudent, int idgroup, Teacher user, java.util.Date start, java.util.Date end) {
		
		System.err.println("getMissQueryContainer(type: " + type + ", idstudent: " + idstudent + ", idgroup: " + idgroup + ", user: " + user + ", start: " + start + ", end: " + end + ")");
		
		QueryContainer querycontainer = null;
		String sqlGroupsOfTeacher=null;
		String sqlStudentsOfGroup=null;
		//String sqlTerms=null;
		String sqlDates=null;
		if (idgroup==0){
			Collection<Group> groups=allGroups(user);
			for (Object g:groups){
				if (sqlGroupsOfTeacher==null){
					sqlGroupsOfTeacher=String.valueOf(((Group) g).getID())+ ",";
				}else{
					sqlGroupsOfTeacher=sqlGroupsOfTeacher + String.valueOf(((Group) g).getID())+ ",";
				}
			}
			sqlGroupsOfTeacher=sqlGroupsOfTeacher.substring(0, sqlGroupsOfTeacher.length()-1);
			sqlGroupsOfTeacher=" AND group_has_student.group_idgroup IN (" + sqlGroupsOfTeacher + ")";
		}else {
			sqlGroupsOfTeacher=" AND group_has_student.group_idgroup=" + idgroup;
		}
		
		if(idstudent==0){
			sqlStudentsOfGroup="";
		}else{
			sqlStudentsOfGroup=" AND miss.student_idstudent="+ idstudent;
		}
		if(start==null || end==null){
			sqlDates="";
		}else{
			sqlDates=" AND date BETWEEN '" +  dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "'" ;
		}

		String sql=" SELECT DISTINCT idmiss,Trim(" + database + ".group.name) AS groupname,Trim(subject.name) AS subjectname,date ,trim(teacher.fullname) AS teachername,classblock.start,classblock.end,student.name,student.surname1,student.surname2 " +
				   " FROM miss" +
				   " INNER JOIN classblock ON classblock_idclassblock = classblock.idclassblock " +
				   " INNER JOIN group_has_student ON classblock.group_idgroup=group_has_student.group_idgroup " +
				   " INNER JOIN teacher ON teacher.idteacher=classblock.teacher_idteacher " +
				   " INNER JOIN student ON miss.student_idstudent=student.idstudent " +
				   " INNER JOIN " + database + ".group ON group_has_student.group_idgroup=" + database + ".group.idgroup " +
				   " INNER JOIN term ON classblock.term_idterm=term.idterm " +
				   " INNER JOIN subject ON classblock.subject_idsubject=subject.idsubject" +
				   " WHERE type=" + type +
				    sqlDates +
				    sqlGroupsOfTeacher +
				    sqlStudentsOfGroup;
		
		System.err.println(sql);
		try {
			
			querycontainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/**
	 * Retorna un QueryContainer con las faltas:
	 *  a) del tipo indicado
	 *  b) entre las fechas indicadas
	 *  c) del estudiante indicado
	 *  
	 * Los campos del QueryContainer retornado son:
	 * +--------+-----------+-----------------------+------------+-----------------------+----------+----------+---------+----------+----------+
	 * | idmiss | groupname | subjectname           | date       | teachername           | start    | end      | name    | surname1 | surname2 |
	 * +--------+-----------+-----------------------+------------+-----------------------+----------+----------+---------+----------+----------+
	 * |      1 | SMX 1r A  | SMX-M1-Muntatge i ... | 2010-09-10 | Roger Borrell Anglada | 13:00:00 | 14:00:00 | Yolanda | Garc�a   | Ros      |
	 * |     27 | SMX 1r A  | SMX-M1-Muntatge i ... | 2010-09-13 | Roger Borrell Anglada | 08:30:00 | 09:30:00 | Yolanda | Garc�a   | Ros      |
	 * |     30 | SMX 1r A  | SMX-M1-Muntatge i ... | 2010-09-13 | Roger Borrell Anglada | 09:30:00 | 10:30:00 | Yolanda | Garc�a   | Ros      |
	 * +--------+-----------+-----------------------+------------+-----------------------+----------+----------+---------+----------+----------+

	 * @param type	Tipo de falta (Miss.MISS | Miss.EXCUSED_MISS | Miss.DELAY | Miss.EXPULSION).
	 * @param start Fecha inicial del rango para buscar faltas
	 * @param end	Fecha final del rango para buscar faltas
	 * @param studentid	ID del estudiante del que se buscarán faltas
	 * @return Un QueryContainer con las faltas encontradas o null si ha fallado algo
	 */
	public QueryContainer getMissQueryContainerByStudent(int type, Date start, Date end, int studentid) {
		Logger.log("DatabaseStorage.getMissQueryContainerByStudent(type: " + type + ", start: " + start + ", end: " + end + ", studentid: " + studentid + ")");
		
		QueryContainer querycontainer = null;
		
		try {
			String sql = "SELECT idmiss, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 FROM miss, " + database + ".group, subject, teacher, classblock, student WHERE miss.type=" + type + " AND date BETWEEN '" + dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "' AND miss.student_idstudent=student.idstudent AND miss.classblock_idclassblock=classblock.idclassblock AND miss.teacher_idteacher=teacher.idteacher AND miss.subject_idsubject=subject.idsubject AND " + database + ".group.idgroup=classblock.group_idgroup  AND subject.idsubject=classblock.subject_idsubject AND teacher.idteacher=classblock.teacher_idteacher AND miss.student_idstudent=" + studentid + " ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			querycontainer = new QueryContainer(sql, con);
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return querycontainer;
	}
	
	/** Retorna un QueryContainer con las faltas de los alumnos del grupo indicado.
	 * 
	 * @param type	Tipo de falta (Miss.MISS | Miss.EXCUSED_MISS | Miss.DELAY | Miss.EXPULSION).
	 * @param start	Fecha inicial del rango para buscar faltas
	 * @param end	Fecha final del rango para buscar faltas
	 * @param groupid	Se retornarán faltas de alumnos que formen parte del grupo especificado, aunque la falta
	 * 					haya sido cometida en una materia de un grupo alternativo.
	 * @return	Un QueryContainer con las faltas encontradas o null si ha fallado algo
	 */
	public QueryContainer getMissQueryContainerByGroup(int type, Date start, Date end, int groupid) {
		Logger.log("DatabaseStorage.getMissQueryContainerByGroup(type: " + type + ", start: " + start + ", end: " + end + ", groupid: " + groupid + ")");
		
		QueryContainer querycontainer = null;
		
		try {
			String sql = "SELECT idmiss, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 FROM miss, " + database + ".group, subject, teacher, classblock, student, group_has_student WHERE miss.type=" + type + " AND date BETWEEN '" + dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "' AND miss.student_idstudent=student.idstudent AND miss.classblock_idclassblock=classblock.idclassblock AND miss.teacher_idteacher=teacher.idteacher AND miss.subject_idsubject=subject.idsubject AND " + database + ".group.idgroup=classblock.group_idgroup  AND subject.idsubject=classblock.subject_idsubject AND teacher.idteacher=classblock.teacher_idteacher AND group_has_student.group_idgroup=" + database + ".group.idgroup AND group_has_student.student_idstudent=student.idstudent AND " + database + ".group.idgroup=" + groupid + " ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			querycontainer = new QueryContainer(sql, con);
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return querycontainer;
	}
	
	/**
	 * Retorna un QueryContainer con las faltas de cualquier alumno.
	 * 
	 * @param type		Tipo de falta (Miss.MISS | Miss.EXCUSED_MISS | Miss.DELAY | Miss.EXPULSION).
	 * @param start		Fecha inicial del rango para buscar faltas
	 * @param end		Fecha final del rango para buscar faltas
	 * @return			Un QueryContainer con las faltas encontradas o null si ha fallado algo
	 */
	public QueryContainer getMissQueryContainerAdminMode(int type, Date start, Date end) {
		Logger.log("DatabaseStorage.getMissQueryContainerAdminMode(type: " + type + ", start: " + start + ", end: " + end + ")");
		
		QueryContainer querycontainer = null;
		
		try {
			String sql = "SELECT idmiss, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 FROM miss, " + database + ".group, subject, teacher, classblock, student WHERE miss.type=" + type + " AND date BETWEEN '" + dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "' AND miss.student_idstudent=student.idstudent AND miss.classblock_idclassblock=classblock.idclassblock AND miss.teacher_idteacher=teacher.idteacher AND miss.subject_idsubject=subject.idsubject AND " + database + ".group.idgroup=classblock.group_idgroup  AND subject.idsubject=classblock.subject_idsubject AND teacher.idteacher=classblock.teacher_idteacher ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			querycontainer = new QueryContainer(sql, con);
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return querycontainer;
	}
	
	/**
	 * Retorna un QueryContainer con las faltas de los alumnos que forman parte de cualquiera de los grupos
	 * de los que el profesor indicado es tutor.
	 * 
	 * @param type			Tipo de falta (Miss.MISS | Miss.EXCUSED_MISS | Miss.DELAY | Miss.EXPULSION).
	 * @param start			Fecha inicial del rango para buscar faltas
	 * @param end			Fecha final del rango para buscar faltas
	 * @param teacherid		ID del profesor, se retornarán faltas de los alumnos que estén en cualquiera de los grupos de los que el profesor es tutor
	 * @return				Un QueryContainer con las faltas encontradas o null si ha fallado algo
	 */
	public QueryContainer getMissQueryContainerByTeacher(int type, Date start, Date end, int teacherid) {
		Logger.log("DatabaseStorage.getMissQueryContainerByTeacher(type: " + type + ", start: " + start + ", end: " + end + ", teacherid: " + teacherid + ")");
		
		QueryContainer querycontainer = null;
		
		try {
			String sql = "SELECT idmiss, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 FROM miss, " + database + ".group, subject, teacher, classblock, student WHERE miss.type=" + type + " AND date BETWEEN '" + dateFormat.format(start) + "' AND '" +  dateFormat.format(end) + "' AND miss.student_idstudent=student.idstudent AND miss.classblock_idclassblock=classblock.idclassblock AND miss.subject_idsubject=subject.idsubject AND " + database + ".group.idgroup=classblock.group_idgroup AND subject.idsubject=classblock.subject_idsubject AND teacher.idteacher=classblock.teacher_idteacher AND " + database + ".group.teacher_idteacher=" + teacherid + " ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			querycontainer = new QueryContainer(sql, con);
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return querycontainer;
	}
	
	public List<Miss> getStudentMisses(int studentid, Date start, Date end, int type) {
		List<Miss> misses = new ArrayList<Miss>();
		
		try {
			String sql = "SELECT idmiss, miss.comment, miss.teacher_idteacher, miss.subject_idsubject, miss.classblock_idclassblock, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 FROM miss, " + database + ".group, subject, teacher, classblock, student WHERE miss.type=" + type + " AND date BETWEEN '" + dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "' AND miss.student_idstudent=student.idstudent AND miss.classblock_idclassblock=classblock.idclassblock AND miss.teacher_idteacher=teacher.idteacher AND miss.subject_idsubject=subject.idsubject AND " + database + ".group.idgroup=classblock.group_idgroup  AND subject.idsubject=classblock.subject_idsubject AND teacher.idteacher=classblock.teacher_idteacher AND miss.student_idstudent=" + studentid + " ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			//System.out.println(sql);
			ResultSet result = statement.executeQuery(sql);
			while(result.next()) {
				//Miss(int id, int type, Date date, String comment, int idstudent, int idteacher, int idsubject, int idclassblock)
				misses.add(new Miss(result.getInt("idmiss"), type, result.getDate("date"),  result.getString("miss.comment"), studentid, result.getInt("miss.teacher_idteacher"), result.getInt("miss.subject_idsubject"), result.getInt("miss.classblock_idclassblock")));
			}
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return misses;
	}
	
	/**
	 * Adds a Teacher to a DDBB. A Teacher gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param teacher with ID 0
     * @return new ID or 0 if not added
	 */
	public int addTeacher(Teacher teacher) {
		int id = 0;
		
		if (teacher.getID()!=0) {
			Logger.log("DatabaseStorage.addTeacher() try to add teacher with no 0 ID\n" + teacher);
			return 0;
		}
		
		try {
			
			int admin = teacher.getAdmin() ? 1 : 0; 
			statement.execute("INSERT INTO teacher VALUES('0', '" + teacher.getLogin() + "', '" + teacher.getPassword() + "', '" + teacher.getFullname() + "', '" + teacher.getEmail() +  "', '" + teacher.getComment() + "', '" + admin + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			teacher.setID(id);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	/** Update the specified teacher value in database. */
	public void updateTeacher(Teacher teacher,Boolean updatepassword) {
		
		try {
			int admin = teacher.getAdmin() ? 1 : 0;
			if (updatepassword){
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', password='" + teacher.getPassword() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
			}else{
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Update the specified teacher value in database. */
	public void updateTeacher(Teacher teacher) {
		
		try {
			int admin = teacher.getAdmin() ? 1 : 0;
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', password='" + teacher.getPassword() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Returns a collection of all teachers in DB*/
	public Collection<Teacher> allTeachers() {
		Vector<Teacher> vector = new Vector<Teacher>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM teacher ORDER BY teacher.fullname;");
			while(result.next()) {
				vector.add(new Teacher(result.getInt("idteacher"), result.getString("login"), result.getString("password"),  result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
	 * Returns a collection of all Groups in DB
	 */
	public List<Group> allGroups() {
		Vector<Group> vector = new Vector<Group>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM " + database + ".group ORDER BY " + database + ".group.name;");
			while(result.next()) {
				vector.add(new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"),  result.getInt("teacher_idteacher")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
    /** Returns a collection of all groups in DB from a Teacher*/
    public List<Group> allGroups(Teacher teacher) {
    		int idteacher=teacher.getID();
            Vector<Group> vector = new Vector<Group>();

            try {
                    ResultSet result = statement.executeQuery("SELECT * FROM " + database + ".group WHERE teacher_idteacher=" + idteacher + " ORDER BY " + database + ".group.name;");
                    while(result.next()) {
                            vector.add(new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"), result.getInt("teacher_idteacher")));
                    }
            } catch (SQLException e) {
                    e.printStackTrace();
            }

            return vector;
    }

	
	/**
	 * Returns specified Teacher object or null
	 * 
	 * @param idteacher
	 * @return
	 */
	public Teacher getTeacher(int idteacher) {
		Teacher teacher = null;
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM teacher WHERE idteacher='" + idteacher + "';");
			while(result.next()) {
				teacher = new Teacher(result.getInt("idteacher"), result.getString("login"), result.getString("password"),  result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return teacher;
	}
	
	/**
	 * Returns a QueryContainer with all teachers
	 *
	 */
	public QueryContainer getTeachers() {
		QueryContainer querycontainer = null;
		try {
			querycontainer = new QueryContainer("SELECT idteacher, login, password, fullname, email, comment, admin FROM teacher ORDER BY teacher.fullname;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/** Deletes the specified teacher */
	public void deleteTeacher(int idteacher) {
		try {
			statement.execute("DELETE FROM teacher WHERE idteacher = " + idteacher + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Deletes the specified miss */
	public void deleteMiss(int type, java.util.Date date, int idstudent, int idclassblock) {
		try {
			statement.execute("DELETE FROM miss WHERE type = " + type + " AND date = '" + dateFormat.format(date) + "' AND student_idstudent = " + idstudent + " AND classblock_idclassblock = " + idclassblock + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns a collection of all classblocks for the specified teacher
	 * on the specified date, a classblock is only valid in a term.
	 * 
	 * The class <code>Classblock</code> has a name only used on the
	 * user interface, not stored in DBMS. The name of all returned classblocks
	 * is: start + " - " + end + " " + group.name + "/" + subject.name 
	 * 
	 * @param idtecher ID of theacher
	 * @param date 	   Date
	 * @return Collection<Classblock> with all valid classblocks for teacher on date
	 */
	public Collection<Classblock> classblocksOfTeacherOnDate(int idteacher, java.util.Date date) {
		Vector<Classblock> vector = new Vector<Classblock>();
		
		try {
			//System.out.println("classblocksOfTeacherOnDate(teacher=" + idteacher + ", date=" + date);
			//System.out.println("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend FROM classblock, term WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.teacher_idteacher=" + idteacher + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "') ORDER BY start;");
			ResultSet result = statement.executeQuery("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.teacher_idteacher=" + idteacher + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject) ORDER BY start;");
			while(result.next()) {
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("term_idterm"), result.getInt("teacher_idteacher"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
//				System.out.println("Bloque: " + block);
				block.setName(timeFormat.format(result.getTime("start")) + "-" + timeFormat.format(result.getTime("end")) + " " + result.getString("groupname") + " / " + result.getString("subjectname"));
				vector.add(block);
			}
		} catch (SQLException e) {
			Logger.log("DatabaseStorage.classblocksOnDate(" + idteacher + ", " + date + ")");
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
	 * Returns a collection of all classblocks of any teacher on the
	 * specified date.
	 * 
	 * The class <code>Classblock</code> has a name only used on the
	 * user interface, not stored in DBMS. The name of all returned classblocks
	 * is: start + " - " + end + " " + group.name + "/" + subject.name 
	 * 
	 * @param idteacher
	 * @param date
	 * @return
	 */
	public Collection<Classblock> classblocksOnDate(java.util.Date date) {
		Vector<Classblock> vector = new Vector<Classblock>();
		
		//Logger.log("classblocksOnDate(" + date + ")");
		try {
			ResultSet result = statement.executeQuery("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, teacher.*, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, teacher, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject AND classblock.teacher_idteacher=teacher.idteacher) ORDER BY start;");
			while(result.next()) {
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("term_idterm"), result.getInt("teacher_idteacher"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				//System.out.println(" -> Bloque: " + block);
				block.setName(timeFormat.format(result.getTime("start")) + "-" + timeFormat.format(result.getTime("end")) + " " + result.getString("groupname") + " / " + result.getString("subjectname") + " (" + result.getString("fullname") + ")");
				vector.add(block);
			}
		} catch (SQLException e) {
			Logger.log("DatabaseStorage.classblocksOfTeachersOnDate(" + date + ")");
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/** 
	 * Returns true if exists a miss for the specified type, date, classblockID
	 * and studentID
	 * 
	 * @param type Type of the miss to look for: Miss.MISS, Miss.EXPULSION, Miss.DELAY, Miss.EXCUSSED_MISS
	 * @param date	Date of miss
	 * @param classblockid	The ID of the classblock
	 * @param studentid The ID of the student
	 * @return true if the specified miss exists
	 */
	public boolean existsMiss(int type, java.util.Date date, int classblockid, int studentid) {
		Boolean exists = false;
	
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM miss WHERE type = " + type + " AND date = '" + dateFormat.format(date) + "' AND classblock_idclassblock = " + classblockid + " AND student_idstudent = " + studentid + ";");
			if(result.next()) exists = true;
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}
	
	/** Returns a string of lowercase hexcodes of SHA-1 hash value for the provided password */
	public String getPasswordHash(String password) {
		return hbinary.marshal( md.digest(password.getBytes()) ).toLowerCase();
	}
	
	/**
	 * Returns a QueryContainer with all groups and a count (as students)
	 *  of group's students.
	 */
	public QueryContainer getGroupsQueryContainer() {
		QueryContainer groupsQueryContainer = null;

		//String sql="SELECT " + database + ".group.*, COUNT(group_has_student.student_idstudent) AS students"
		//	 + " FROM " + database + ".group LEFT JOIN group_has_student ON " + database + ".group.idgroup = group_has_student.group_idgroup "
		//	 +  "GROUP BY " + database + ".group.idgroup;";
		//NEW SQL TABLE GROUP WITH MANAGER TEACHER
		String sql = "SELECT " + database + ".group.*, COUNT(group_has_student.student_idstudent) AS students,IFNULL(teacher.fullname,'" + locale.getString("GROUPMANAGERLAYOUT_COMBOBOX_DEFAULTCAPTION") + "') as tutor FROM " + database + ".group LEFT JOIN group_has_student ON " + database + ".group.idgroup = group_has_student.group_idgroup LEFT JOIN teacher ON teacher_idteacher = teacher.idteacher GROUP BY " + database + ".group.idgroup ORDER BY name;";

		try {

			groupsQueryContainer = new QueryContainer(sql, con);

			//groupsQueryContainer = new QueryContainer("SELECT " + database + ".group.*, COUNT(group_has_student.student_idstudent) AS students FROM " + database + ".group LEFT JOIN group_has_student ON " + database + ".group.idgroup = group_has_student.group_idgroup GROUP BY " + database + ".group.idgroup;", con);

		} catch (SQLException e) {
			e.printStackTrace();
		}
		return groupsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with groups of student
	 */
	public QueryContainer getGroupsOfStudentQueryContainer(Integer idStudent) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT idgroup,name,comment from " + database + ".group,group_has_student";
			sql = sql + " WHERE idgroup=group_idgroup";
			sql = sql + " AND student_idstudent=" + idStudent + " ORDER BY name;";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with groups that not have student
	 */
	public QueryContainer getGroupsHaveNotStudentGroupQueryContainer(Integer idStudent) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT DISTINCT idgroup,name,comment from " + database + ".group";
			sql = sql + " WHERE idgroup NOT IN (";
			sql = sql + " SELECT DISTINCT group_idgroup FROM group_has_student";
			sql = sql + " WHERE student_idstudent=" + idStudent + ") ORDER BY name;";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns specified group or null if not exists.
	 * 
	 * @param idgroup
	 * @return specified group or null if not exists.
	 */
	public Group getGroup(int idgroup) {
		Group group = null;
		ResultSet result;
		
		try {
			result = statement.executeQuery("Select * from " + database + ".group where idgroup=" + idgroup + ";");
			if(result.next()) group = new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"), result.getInt("teacher_idteacher"));

		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return group;
	}
	
	/**
	 * 
	 * Gets the name of a group.
	 * 
	 * @param idgroup ID of group to look for
	 * @return the name of group or null if not in DB
	 */
	public String getGroupName(int idgroup) {
		
		ResultSet result;
		String name = null;
		try {
			result = statement.executeQuery("Select name from " + database + ".group where idgroup=" + idgroup + ";");
			if(result.next()) name = result.getString("name");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return name;
	}
	
	/**
	 * Gets the <code>Teacher</code> assigned to a group or null.
	 * @param idgroup
	 * @return Teacher object 
	 */
	public Teacher getGroupTeacher(int idgroup) {
		Group group = getGroup(idgroup);
		if (group == null) return null;
		
		Teacher teacher = getTeacher(group.getIDTeacher());
		if (teacher == null) return null;
		
		return teacher;
	}
	
	/**
	 * Returns a QueryContainer with all students
	 */
	public QueryContainer getStudentsQueryContainer() {
		QueryContainer studentsQueryContainer = null;
		try {
			studentsQueryContainer = new QueryContainer("SELECT idstudent, surname1, surname2, name, email, smsphone FROM student ORDER BY surname1, surname2, name;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Retorna una colección de faltas para el estudiante indicado.
	 * 
	 * @param studentid
	 * @param start
	 * @param end
	 * @return
	 */
	public ArrayList<Miss> getStudentMisses(int studentid, int classblockid, Date date) {
		//Logger.log("DatabaseStorage.getStudentMisses(studentid: " + studentid + ", start: " + start + ", date: " + end + ")");
		
		ArrayList<Miss> list = new ArrayList<Miss>();
		
		try {
			//TODO Esta consulta selecciona más columnas de las necesarias
			String sql = "SELECT miss.*, TRIM(" + database + ".group.name) AS groupname, TRIM(subject.name) AS subjectname, date, TRIM(teacher.fullname) AS teachername, classblock.start, classblock.end, student.name, student.surname1, student.surname2 " +
					"FROM miss, " + database + ".group, subject, teacher, classblock, student " +
					"WHERE date='" + dateFormat.format(date) + "' " +
					"AND miss.student_idstudent=student.idstudent " +
					"AND miss.classblock_idclassblock=classblock.idclassblock " +
					"AND miss.classblock_idclassblock=" + classblockid + " " +
					"AND miss.teacher_idteacher=teacher.idteacher " +
					"AND miss.subject_idsubject=subject.idsubject " +
					"AND " + database + ".group.idgroup=classblock.group_idgroup  " +
					"AND subject.idsubject=classblock.subject_idsubject " +
					"AND teacher.idteacher=classblock.teacher_idteacher " +
					"AND miss.student_idstudent=" + studentid + " ORDER BY date, start, surname1, surname2, name, groupname, subjectname, teachername;";
			ResultSet result = statement.executeQuery(sql);
			while(result.next()) {
				list.add(new Miss(result.getInt("idmiss"), result.getInt("type"), result.getDate("date"), result.getString("comment"), result.getInt("student_idstudent"), result.getInt("teacher_idteacher"), result.getInt("subject_idsubject"), result.getInt("classblock_idclassblock")));
			}
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return list;
	}
	
	/**
	 * Returns a QueryContainer with all terms
	 */
	public QueryContainer getTerms() {
		QueryContainer termsQueryContainer = null;
		try {
			termsQueryContainer = new QueryContainer("SELECT * FROM term ORDER BY term.name;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return termsQueryContainer;
	}
	
	/**
	 *  Returns a collection of all terms in DB
	 * */
	public Collection<Term> allTerms() {
		Vector<Term> vector = new Vector<Term>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM term ORDER BY term.name;");
			while(result.next()) {
				vector.add(new Term(result.getInt("idterm"), result.getString("name"), result.getString("comment"),result.getDate("start"),result.getDate("end")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}

	/**
	 * Adds a Term to a DDBB. A Term gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param term with ID 0
     * @return new ID or 0 if not added
	 */
	public Integer addTerm(Term term) {
		int id = 0;
		
		if (term.getID()!=0) {
			Logger.log("DatabaseStorage.addTerm() try to add term with no 0 ID\n" + term);
			return 0;
		}
		
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			statement.execute("INSERT INTO term VALUES(" +
					"'0', '" + term.getName() + "', '" + term.getComment() 
					+ "', '" + sdf.format(term.getStart()) + "', '" + sdf.format(term.getEnd()) + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			term.setID(id);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	
	/**
	 * Update term information
	 */
	public Integer updateTerm(Term term) {
		
		Integer id=term.getID();
		String name=term.getName();
		String comment=term.getComment();
		java.util.Date start=term.getStart();
		java.util.Date end=term.getEnd();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String sql="UPDATE term SET name='" + name +"' , comment='"+ comment + "'," + 
        	"start='" +  sdf.format(start) +"',end='"+ sdf.format(end) +
        	"' WHERE idterm=" + id + ";";
        int res=0;
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }

        return res;
	}
	
	/** Deletes the specified term */
	public void deleteTerm(int idterm) {
		try {
			statement.execute("DELETE FROM term WHERE idterm = " + idterm + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/** Returns a collection of subjects*/
	public Collection<Subject> allSubjects() {
		Vector<Subject> vector = new Vector<Subject>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM subject ORDER BY subject.name;");
			while(result.next()) {
				vector.add(new Subject(result.getInt("idsubject"), result.getString("name"), result.getString("comment")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}

	public QueryContainer getSubjects() {
		QueryContainer querycontainer = null;
		try {
			querycontainer = new QueryContainer("SELECT * FROM subject ORDER BY subject.name;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/**
	 * Adds a Subject to a DDBB. A Subject gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param subject with ID 0
     * @return new ID or 0 if not added
	 */
	public int addSubject(Subject subject) {
		int id = 0;
		
		if (subject.getID()!=0) {
			Logger.log("DatabaseStorage.addSubject() try to add subject with no 0 ID\n" + subject);
			return 0;
		}
		
		try { 
			statement.execute("INSERT INTO subject VALUES('0', '" +  subject.getName() + "', '" + subject.getComment() +  "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id =result.getInt("ID");
			subject.setID(id);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	
	/**
	 * Refresh table data from a QueryContainer. The previous content of table
	 * is removed and new data is loaded from the provided QueryContainer.
	 * 
	 * The value of the first property of QueryContainer will be used as
	 * Item ID in table.
	 * 
	 * @param qc 			A QueryContainer with data to load
	 * @param table			Table to refresh
	 */
	public void refreshTable(QueryContainer qc, Table table) {
		table.removeAllItems();
		
		int qcsize = qc.size();
		if (qcsize == 0) {
			Logger.log("DatabaseStorage.refreshTable() empty query container");
			return;
		}
		
		List<Object> qcpropertyIds = new ArrayList<Object>(qc.getContainerPropertyIds());
		int propertiesnumber = qcpropertyIds.size();
		if (propertiesnumber <= 1) {
			Logger.log("DatabaseStorage.refreshTable() query container needs more than one property");
			return;
		}
		
		List<Object> tablepropertyIds = new ArrayList<Object>(table.getContainerPropertyIds());
		
		for(Object o : tablepropertyIds) {
			if(!qcpropertyIds.contains(o)) {
				Logger.log("DatabaseStorage.refreshTable()");
				Logger.log(" - Table property not found in query container");
				Logger.log(" - PropertyID: " + o);
				return;
			}
		}
		
//		System.out.println("qcpropertyIds: " + qcpropertyIds);
//		System.out.println("tablepropertyIds: " + tablepropertyIds);
		
		for(int i=1; i<=qcsize; i++) {
			Item dataItem = qc.getItem(i);
			Item tableItem = table.addItem(dataItem.getItemProperty(qcpropertyIds.get(0)).getValue());
			if (tableItem == null) {
				Logger.log("DatabaseStorage.refreshTable() couldn't add new table's item");
			}
			for(int j=1; j<propertiesnumber; j++) {
				Object pid = qcpropertyIds.get(j);
				//System.out.println("DatabaseStorage.refreshTable() pid: " + pid);
				try {
					tableItem.getItemProperty(pid).setValue(dataItem.getItemProperty(pid).getValue());
				} catch(Exception e) {
					Logger.log("DatabaseStorage.refreshTable():");
					Logger.log(" pid: " + pid);
					e.printStackTrace();
				}
			}
		}

	}
	


	/**
	 * Adds a Student to a DDBB. A Student gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param student with ID 0
     * @return new ID or 0 if not added
	 */
	public int addStudent(Student student) {
			
			int id = 0;
			
			if (student.getID()!=0) {
				Logger.log("DatabaseStorage.addStudent() try to add student with no 0 ID\n" + student);
				return 0;
			}
			
			try {
				//Añadido campo photoid para introduccion de blob
				PreparedStatement st = con.prepareStatement("INSERT INTO student(surname1,surname2,name,email,smsphone,photoid) " 
						+ " VALUES('" 
						+ student.getSurname1() + "','" 
						+ student.getSurname2() + "','"
						+ student.getName() + "','"
						+ student.getEmail() + "','"
						+ student.getSmsphone()+ "'," 
						+ "?);");
	//---->>>>
	//			Creacion de fileImputStram recojemos la array de bytes de ese student y lo intoducimos en la BD
				
				ByteArrayInputStream bais = new ByteArrayInputStream(student.getBytePhotoArray());
				st.setBlob(1,bais);
	
				st.execute();
				st.close();
	
	
				
				ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
				result.next();
				id = result.getInt("ID");
				student.setID(id);
				
			} catch (SQLException e) {
				e.printStackTrace();
			}
			
			return id;
		}
		
	//----------------_>>
	
	
/////---------------------------->>>>>>>>>>>>>>>>>>>
	public int addContact(StudentContacts contact, int idStudent) {
			
			int id = 0;
			
			if (contact.getIdContact()!=0) {
				Logger.log("DatabaseStorage.addStudent() try to add contact with no 0 ID\n" + contact);
				return 0;
			}
			
			try {
				
				statement.executeUpdate("INSERT INTO contacts(name,surname1,surname2,address,phone,smsphone,parent) " 
						+ " VALUES('" 
						+ contact.getName() + "','" 
						+ contact.getSurname1() + "','" 
						+ contact.getSurname2() + "','"
						+ contact.getAddress() + "','"
						+ contact.getPhone() + "','"
						+ contact.getSmsphone() + "','"
						+ contact.getParent() + "');");
				ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
				result.next();
				id = result.getInt("ID");
				contact.setIdContact(id);
				
				
			
			} catch (SQLException e) {
				System.err.println("DatabaseStorage.adContact-> Intro Contacts table ----->>>  "+e.toString());
			}
			
		try {
				
				statement.executeUpdate("INSERT INTO studentcontacts(idstudent,idcontact) " 
						+ " VALUES('" 
						+ idStudent + "','" 
						+ contact.getIdContact() + "');");
				ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
				result.next();
				id = result.getInt("ID");
				contact.setIdContact(id);
				
				
			
			} catch (SQLException e) {
				System.err.println("DatabaseStorage.adContact-> Intro StudentContacts table ----->>>  "+e.toString());
			}
			
			
			return id;
		}
		
	/////---------------------------->>>>>>>>>>>>>>>>>>>
	
	/**
	 * Metodo retorna estudiante de la base de datos a traves de su id
	 * @param idstudent
	 * @return
	 */
	public Student getStudent(int idstudent) {
		Student student = null;
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM student WHERE idstudent='" + idstudent + "';");
			while(result.next()) {
				student = new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"),  result.getString("name"), result.getString("email"), result.getString("smsphone"),result.getBytes("photoid"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return student;
	}
	
//----------------_>>
	
	
	/**
	 * Adds the specified studentid to the group with groupid.
	 * 
	 * @param studentid	ID of student
	 * @param groupid	ID of group 
	 */
	public void addStudentToGroup(int studentid, int groupid) {
		try {
			statement.executeUpdate("INSERT INTO group_has_student VALUES('" + groupid +"','" + studentid + "');");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Update values Subject
	 */
	public int updateSubject(Subject subject) {
		Integer id=subject.getID();
		String name=subject.getName();
		String comment=subject.getComment();
        String sql="UPDATE subject SET name='" + name +"' , comment='"+ comment + 
        	"' WHERE idsubject=" + id + ";";
        int res=0;
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }
        return res;
	}
	
    /** Deletes the specified subject */
    public void deleteSubject(Integer idsubject ) {
            try {
                    statement.execute("DELETE FROM subject WHERE idsubject = " + idsubject +  ";");
            } catch (SQLException e) {
                    e.printStackTrace();
            }
    }

    /** Update Student Information*/
    public int updateStudent(Student student){
        	int res=0;
    		Integer id=student.getID();
    		String name=student.getName();
    		String surname1=student.getSurname1();
    		String surname2=student.getSurname2();
    		String email=student.getEmail();
    		String smsphone=student.getSmsphone();
            String sql="UPDATE student SET name='" + name +"' ," 
            		+ " surname1='"+ surname1 + "',"
            		+ " surname2='"+ surname2 + "',"
            		+ " email='"+ email + "',"
            		+ " smsphone='"+ smsphone + "'"
            		+ " WHERE idstudent=" + id + ";";
            try{
                    res = statement.executeUpdate(sql);
            } catch (SQLException e) {
                    e.printStackTrace();
            }

            return res;
    }
    
    /**
	 * Delete Subject
	 */
	public void deleteSubject(int idSubject){
		try {
			statement.execute("DELETE FROM subject WHERE idsubject="+ idSubject); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
    
	/**
	 * Delete Student
	 */
	public void deleteStudent(Student student){
		try {
			statement.execute("DELETE FROM student WHERE idstudent="+ student.getID()); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Delete Student to Group
	 */
	public void deleteStudentFromGroup(Integer idgroup,Integer idstudent){
		try {
			statement.execute("DELETE FROM group_has_student WHERE group_idgroup=" + idgroup + " AND student_idstudent=" + idstudent ); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Returns a QueryContainer with students of group
	 */
	public QueryContainer getStudentsOfGroupQueryContainer(Integer idgroup) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql=" SELECT idstudent,surname1,surname2,name,email,smsphone from student s,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup + " ORDER BY surname1, surname2, name;";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a Collection with students of group
	 * If idgroup==0 return all students
	 */
	public List<Student> getStudentsCollection(int idgroup) {
		Vector<Student> vector = new Vector<Student>();
		String sql="";
		if (idgroup==0){
			sql=" SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone";
			sql = sql + " FROM student,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			//sql = sql + " AND group_idgroup<>" + idgroup;
			sql = sql + " ORDER BY surname1, surname2, name;";
			
		}else{
			sql=" SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone";
			sql = sql + " FROM student,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup;
			sql = sql + " ORDER BY surname1, surname2, name;";
		
		}

		try {
			ResultSet result = statement.executeQuery(sql);
			while(result.next()) {
				String email = result.getString("email");
				//if(result.wasNull()) System.out.println("EMAIL NULO: " + email);
				vector.add(new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"),result.getString("name"), email,result.getString("smsphone")));
			}
		} catch (SQLException e) {
				e.printStackTrace();
			}
			
		return vector;
	}
	
	/**
	 * Retorna una colección de estudiantes que pertenecen al grupo especificado. Si
	 * idgroup==0 entonces retorna la colección de todos los estudiantes que pertenecen
	 * a alguno de los grupos de los que idteacher es tutor.
	 * 
	 * @param idgroup Grupo del que se obtendrán los estudiantes. Si es 0 se obtienen
	 *                los estudiantes de todos los grupos del tutor indicado.
	 * @param idteacher Tutor de los grupos
	 * @return La lista de estudiantes del grupo o de todos los grupos de los que
	 * 		   el profesor es tutor.
	 */
	public List<Student> getStudentsCollection(int idgroup, int idteacher) {
		Vector<Student> vector = new Vector<Student>();
		String sql="";
		if (idgroup==0){
			sql="SELECT DISTINCT student.idstudent, student.surname1, student.surname2, student.name, student.email, student.smsphone ";
			sql = sql + "FROM student, group_has_student, " + database + ".group ";
			sql = sql + "WHERE student.idstudent=group_has_student.student_idstudent ";
			sql = sql + "AND group_has_student.group_idgroup=" + database + ".group.idgroup ";
			sql = sql + "AND " + database + ".group.teacher_idteacher=" + idteacher + " ";
			//sql = sql + " AND group_idgroup<>" + idgroup;
			sql = sql + "ORDER BY surname1, surname2, name;";
			
		}else{
			sql="SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone ";
			sql = sql + " FROM student,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup;
			sql = sql + " ORDER BY surname1, surname2, name;";
		
		}

		try {
			ResultSet result = statement.executeQuery(sql);
			while(result.next()) {
				String email = result.getString("email");
				//if(result.wasNull()) System.out.println("EMAIL NULO: " + email);
				vector.add(new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"),result.getString("name"), email,result.getString("smsphone")));
			}
		} catch (SQLException e) {
				e.printStackTrace();
			}
			
		return vector;
	}
	
	/**
	 * Returns a QueryContainer with contact information of students of group.
	 * 
	 * @author Xavier Junyent <xavierjunyent@yahoo.es>
	 */
	public QueryContainer getStudentsContactInfoQueryContainer(int idgroup) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql=" SELECT idstudent,CONCAT(surname1,' ', surname2,', ',name)as fullname,email,smsphone from student s,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup + "";
			sql = sql + " ORDER BY surname1, surname2, name;";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with students of group
	 */
	public QueryContainer getStudentsNotInGroupQueryContainer(Integer idgroup) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone from student";
			sql = sql + " WHERE idstudent NOT IN (";
			sql = sql + " SELECT DISTINCT student_idstudent FROM group_has_student";
			sql = sql + " WHERE group_idgroup=" + idgroup + ") ORDER BY surname1, surname2, name;";
			
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Removes the specified studentid from the group with groupid.
	 * 
	 * @param studentid	ID of student
	 * @param groupid	ID of group 
	 */
	public void removeStudentFromGroup(int studentid, int groupid) {
		try {
			statement.executeUpdate("DELETE FROM group_has_student WHERE (group_has_student.group_idgroup='" + groupid + "' AND group_has_student.student_idstudent='" + studentid + "');");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	//
	// Código para generar los informes y las notificaciones SMS/Mail
	// 
	
	/**
	 * Returns a <code>ResultSet</code> with count of misses in a given group.
	 * 
	 * The result set has columns:
	 * | idstudent | surname1 | surname2 | name     | smsphone | email               | misscount | excusedcount | delaycount | expulsioncount |
	 * <pre>
	 * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
     * | idstudent | surname1 | surname2 | name     | smsphone | email               | misscount | excusedcount | delaycount | expulsioncount |
     * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
     * |         1 | student1 | student1 | student1 | 12345    | student1@server.com |         3 |            0 |          0 |              0 |
     * |         2 | student2 | student2 | student2 | 12345    | student2@server.com |         0 |            1 |          0 |              0 |
     * |         3 | student3 | student3 | student3 | 12345    | student3@server.com |         2 |            1 |          0 |              0 |
     * |         5 | student5 | student5 | student5 | 12345    | student5@server.com |         1 |            1 |          0 |              0 |
     * |         6 | student6 | student6 | student6 | 12345    | student1@server.com |         1 |            0 |          0 |              0 |
     * |         7 | student7 | student7 | student7 | 12345    | student2@server.com |         0 |            0 |          3 |              0 |
     * |         8 | student8 | student8 | student8 | 12345    | student3@server.com |         0 |            0 |          2 |              0 |
     * |         9 | student9 | student9 | student9 | 12345    | student4@server.com |         0 |            0 |          1 |              2 |
     * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
	 * </pre>
	 * 
	 * @param idgroup ID of group to report
	 * @param dateA   Begin date for miss count
	 * @param dateB   End date for miss count
	 * 
	 * @return ResultSet with result data or null
	 */
	public ResultSet studentReport1(int idgroup, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			String sql = "SELECT student.idstudent, student.surname1, student.surname2, student.name, student.smsphone, student.email, sum(if (miss.type=1,1,0)) 'misscount', sum(if (miss.type = 2,1,0)) 'excusedcount', sum(if (miss.type =3,1,0)) 'delaycount', sum(if (miss.type = 4,1,0)) 'expulsioncount' FROM student INNER JOIN miss ON miss.student_idstudent=student.idstudent INNER JOIN group_has_student ON group_has_student.student_idstudent=student.idstudent WHERE group_idgroup=" + idgroup + " AND miss.date BETWEEN '" + dateFormat.format(dateA) + "' AND '" + dateFormat.format(dateB) + "' GROUP BY student.idstudent ORDER BY student.surname1, student.surname2, student.name;";
			result = statement.executeQuery(sql);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	       
	    return result;
	}
	
	/**
	 * 
	 *
	 * 
	 * @param idgroup
	 * @param dateA
	 * @param dateB
	 * @return
	 */
	public ResultSet studentReport2(int idgroup, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT student.idstudent, student.surname1, student.surname2, student.name, subject.name 'subject',sum(if (miss.type=1,1,0)) 'misscount', sum(if (miss.type = 2,1,0)) 'excusedcount',         sum(if (miss.type =3,1,0)) 'delaycount',sum(if (miss.type = 4,1,0)) 'expulsioncount'   FROM   student INNER JOIN miss ON miss.student_idstudent=student.idstudent INNER JOIN group_has_student ON group_has_student.student_idstudent=student.idstudent INNER JOIN subject on miss.subject_idsubject=subject.idsubject WHERE group_idgroup= " + idgroup + " AND miss.date BETWEEN '" + dateFormat.format(dateA) + "'AND '" + dateFormat.format(dateB) + "' GROUP BY student.idstudent ORDER BY student.surname1, student.surname2, student.name, subject_idsubject;");
		} catch (SQLException e) {
			e.printStackTrace();
		}
				
		return result;
	}
	
	/**
	 * 
	 *
	 * 
	 * @param idgroup
	 * @param dateA
	 * @param dateB
	 * @return
	 */
	public ResultSet studentReport4(int idgroup, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT student.name, student.surname1, student.surname2, miss.date, classblock.day_of_week, DATE_FORMAT(classblock.start, '%H:%i') AS start, DATE_FORMAT(classblock.end, '%H:%i') AS end, subject.name AS subject, miss.type FROM student, group_has_student, miss, classblock, subject WHERE group_has_student.group_idgroup=" + idgroup + " AND miss.date BETWEEN '" + dateFormat.format(dateA) + "' AND '" + dateFormat.format(dateB) + "' AND group_has_student.student_idstudent=student.idstudent AND miss.student_idstudent=student.idstudent AND classblock.idclassblock=miss.classblock_idclassblock AND subject.idsubject=classblock.subject_idsubject ORDER BY student.surname1, student.surname2, student.name, miss.date, classblock.start, miss.type;");
		} catch (SQLException e) {
			e.printStackTrace();
		}
				
		return result;
	}
	
	/**
	 * Retorna la suma de faltas por alumno para una fecha a partir de una hora. Dado un grupo, un tipo de falta, una fecha
	 * y una hora, se retorna un listado con la suma de faltas (del tipo indicado) de cada alumno del grupo posteriores a la
	 * hora indicada.
	 * 
	 * Este método se utiliza en SMSGroupNotifierv2
	 * 
	 * @param idgroup
	 * @param type
	 * @param dateA
	 * @param time
	 * @return ResultSet con la suma de faltas posteriores a la hora indicada.
	 */
	public ResultSet studentReport5(int idgroup, int type, Date dateA, Time time) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT COUNT(*) AS misscount, student.idstudent, student.surname1, student.surname2, student.name, student.smsphone, student.email FROM student, " + database + ".group, group_has_student, miss, classblock WHERE " + database + ".group.idgroup=" + idgroup + " AND student.idstudent = group_has_student.student_idstudent AND group_has_student.group_idgroup = " + database + ".group.idgroup AND student.idstudent = miss.student_idstudent AND miss.type = " + type + " AND miss.date='" + dateFormat.format(dateA) + "' AND miss.classblock_idclassblock = classblock.idclassblock AND classblock.start >= '" + time + "' GROUP BY idstudent;");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return result;
	}
	
	/**
	 * Retorna la suma de faltas de un alumno en el rango de fechas proporcionado.
	 * 
	 * Este método se utiliza en SMSGroupNotifierv2
	 * 
	 * @param idgroup
	 * @param type
	 * @param dateA
	 * @param dateB
	 * @return
	 */
	public ResultSet studentReport5(int idgroup, int type, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT COUNT(*) AS misscount, student.idstudent, student.surname1, student.surname2, student.name, student.smsphone, student.email FROM student, " + database + ".group, group_has_student, miss, classblock WHERE " + database + ".group.idgroup=" + idgroup + " AND student.idstudent = group_has_student.student_idstudent AND group_has_student.group_idgroup = " + database + ".group.idgroup AND student.idstudent = miss.student_idstudent AND miss.type = " + type + " AND miss.date BETWEEN '" + dateFormat.format(dateA) + "' AND '" + dateFormat.format(dateB) + "' AND miss.classblock_idclassblock = classblock.idclassblock GROUP BY idstudent;");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return result;
	}
	
	/**
	 * Reports a count of misses for a student.
	 * 
	 * mysql> SELECT teacher_idteacher, subject_idsubject, COUNT(IF(type=1,1,null)) AS miss, COUNT(IF(type=2,1,null)) AS excused, COUNT(IF(type=3,1,null)) AS delay, COUNT(IF(type=4,1,null)) as expulsion, subject.name AS subject_name FROM miss, subject WHERE student_idstudent=28 AND subject_idsubject=subject.idsubject AND date BETWEEN "2010-08-01" AND "2010-08-30" GROUP BY subject_idsubject ORDER BY subject_name;
+-------------------+-------------------+------+---------+-------+-----------+--------------+
| teacher_idteacher | subject_idsubject | miss | excused | delay | expulsion | subject_name |
+-------------------+-------------------+------+---------+-------+-----------+--------------+
|                 1 |                 8 |    1 |       1 |     0 |         2 | F�sica       |
|                 1 |                10 |    2 |       1 |     0 |         0 | Literatura   |
|                 1 |                 7 |    1 |       1 |     1 |         0 | Matem�ticas  |
|                 1 |                 9 |    1 |       1 |     0 |         0 | Qu�mica      |
+-------------------+-------------------+------+---------+-------+-----------+--------------+
4 rows in set (0.00 sec)

mysql>
	 * 
	 * @param idstudent
	 * @param dateA
	 * @param dateB
	 * @return
	 */
	public ResultSet reportStudentMissCount(int idstudent, Date dateA, Date dateB) {
		
		ResultSet result = null;
		
		try {
			String sql = "SELECT teacher_idteacher, subject_idsubject, COUNT(IF(type=1,1,null)) AS miss, COUNT(IF(type=2,1,null)) AS excused, COUNT(IF(type=3,1,null)) AS delay, COUNT(IF(type=4,1,null)) as expulsion, subject.name AS subject_name FROM miss, subject WHERE student_idstudent=" + idstudent + " AND subject_idsubject=subject.idsubject AND date BETWEEN '" + dateFormat.format(dateA) + "' AND '" + dateFormat.format(dateB) + "' GROUP BY subject_idsubject ORDER BY subject_name;";
			//System.out.println(sql);
			result = statement.executeQuery(sql);
		} catch (SQLException e) {
			e.printStackTrace();
		}
			
		return result;
	}
	
	/**
	 * Returns a classblock by id.
	 * 
	 * @param id
	 * @return classblock or null;
	 */
	public Classblock getClassblock(int id) {
		Classblock cb = null;
		
		try {
			String sql = "SELECT * FROM classblock WHERE classblock.idclassblock=" + id + ";";
			ResultSet result = statement.executeQuery(sql);
			if(result.next()) {
				// (int id, int idterm, int idteacher, int idsubject, int idgroup, int day_of_week, Time start, Time end)
				cb = new Classblock(result.getInt("idclassblock"), result.getInt("term_idterm"), result.getInt("teacher_idteacher"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
			} else return null;
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return cb;
	}
	
	/**
	 * Returns a subject by id or null if not exists.
	 * @param id
	 * @return subject or null
	 */
	public Subject getSubject(int id) {
		Subject subject = null;
		
		try {
			String sql = "SELECT * FROM subject WHERE subject.idsubject=" + id + ";";
			ResultSet result = statement.executeQuery(sql);
			if(result.next()) {
				subject = new Subject(id, 0, result.getString("name"), result.getString("comment"));
			} else return null;
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return subject;
	}

	/**
	 * Returns a QueryContainer with Classblocks
	 * // It's a name composed by time + group.name + " / " + subject.name
	 * */
    public QueryContainer getClassblocks() {
        QueryContainer querycontainer = null;
        ResourceBundle locale = AttendancemanagerApplication.locale;
        String sql;
                sql="SELECT classblock.idclassblock," 
                + "(CASE classblock.day_of_week "
                + " WHEN 1 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SUNDAY")+"'"
                + " WHEN 2 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_MONDAY")+"'"
                + " WHEN 3 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_TUESDAY")+"'"
                + " WHEN 4 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_WEDNESDAY")+"'"
                + " WHEN 5 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_THURSDAY")+"'"
                + " WHEN 6 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_FRIDAY")+"'"
                + " WHEN 7 THEN "  + "'"+ locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SATURDAY")+"'"
                + " END) AS dayofweekstring"
                + " , classblock.day_of_week, classblock.teacher_idteacher, classblock.subject_idsubject, classblock.group_idgroup, classblock.term_idterm, Trim(teacher.fullname) AS teachername, Trim(term.name) as termname, Trim(" + database + ".group.name) as groupname, Trim(subject.name) AS subjectname, classblock.start, classblock.end" 
                + " FROM classblock "
                + " INNER JOIN teacher ON classblock.teacher_idteacher=teacher.idteacher"
                + " INNER JOIN term ON classblock.term_idterm=term.idterm"
                + " INNER JOIN " + database + ".group ON classblock.group_idgroup=" + database + ".group.idgroup"
                + " INNER JOIN subject ON classblock.subject_idsubject=subject.idsubject"
                + " ORDER BY classblock.idclassblock";

        try {
                querycontainer = new QueryContainer(sql, con);;
        } catch (SQLException e) {
                e.printStackTrace();
        }
        return querycontainer;
    }

    /**
     * Adds a Classblock to a DDBB. A Classblock gets one ID when is stored
     * in DDBB for first time, previously it must have ID 0.
     * 
     * @param classblock with ID 0
     * @return new ID or 0 if not added
     */
    public int addClassblock(Classblock classblock){
		int id=0;
		
		if (classblock.getID()!=0) {
			Logger.log("DatabaseStorage.addClassblock() try to add classblock with no 0 ID\n" + classblock);
			return 0;
		}
		
		try {
			
			SimpleDateFormat sdf= new SimpleDateFormat("HH:mm");
			statement.executeUpdate("INSERT INTO classblock  " 
					+ " VALUES(" 
					+ 0 + ",'"
					+ classblock.getIDTeacher() + "','" 
					+ classblock.getIDSubject()+ "','"
					+ classblock.getIDgroup()+ "','"
					+ classblock.getIDTerm()+ "','"
					+ classblock.getDay_of_week() + "','"
					+ sdf.format(classblock.getStart()) + "','"
					+ sdf.format(classblock.getEnd()) + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			classblock.setID(id);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	/**
	 * Update term information
	 */
	public Integer updateClassblock(Classblock classblock) {
		Integer idClassblock=classblock.getID();
		Integer idTeacher =classblock.getIDTeacher();
		Integer idTerm =classblock.getIDTerm();
		Integer idGroup=classblock.getIDgroup();
		Integer idSubject=classblock.getIDSubject();
		Integer idDayofweek=classblock.getDay_of_week();
		java.util.Date hourStart=classblock.getStart();
		java.util.Date hourEnd=classblock.getEnd();
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        String sql="UPDATE classblock SET "
        	       + " teacher_idteacher=" + idTeacher + "," 
        	       + " subject_idsubject="+ idSubject + ","
        	       + " group_idgroup="+ idGroup + ","
        	       + " term_idterm="+ idTerm + ","
        	       + " day_of_week="+ idDayofweek + ","
        	       + " start='" +  sdf.format(hourStart)+ "',"
        	       + " end='"+ sdf.format(hourEnd) + "'"
        	       + " WHERE idclassblock=" + idClassblock + ";";
        int res=0;
        
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }

        return res;
	}
	/**
	 * Delete Classblock
	 */
	public void deleteClassblock(Integer idclassblock){
		try {
			statement.execute("DELETE FROM classblock WHERE idclassblock="+ idclassblock); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns a list with all classblocks of idgroup on date. All classblock
	 * properties are initialized from database except name.
	 * 
	 * @param idgroup
	 * @param date
	 * @return List of group classblocks on date, maybe zerosized.
	 */
	public List<Classblock> getClassblocksOnDate(int idgroup, Date date){
		Vector<Classblock> vector = new Vector<Classblock>();
		
		try {
			//System.out.println("DatabaseStorage.getClassblocksOnDate(idgroup=" + idgroup + ", date=" + date);
			//System.out.println("SELECT * FROM classblock, term WHERE classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND term_idterm=idterm AND group_idgroup=" + idgroup + " AND '" + dateFormat.format(date) + "' BETWEEN term.start AND term.end;");
			ResultSet result = statement.executeQuery("SELECT * FROM classblock, term WHERE classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND term_idterm=idterm AND group_idgroup=" + idgroup + " AND '" + dateFormat.format(date) + "' BETWEEN term.start AND term.end ORDER BY classblock.start;");
			/*
			System.out.println("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.group_idgroup=" + idgroup + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject) ORDER BY start;");
			ResultSet result = statement.executeQuery("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.group_idgroup=" + idgroup + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject) ORDER BY start;");
			*/
			while(result.next()) {
				//Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("teacher_idteacher"), result.getInt("term_idterm"),result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				//System.out.println("Classblock: " + block);
				//block.setName(timeFormat.format(result.getTime("start")) + "-" + timeFormat.format(result.getTime("end")) + " " + result.getString("groupname") + " / " + result.getString("subjectname"));
				vector.add(block);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
	 * Retorna la lista de Classblock de estudiante en la fecha indicada.
	 * 
	 * @param idstudent
	 * @param date
	 * @return
	 */
	public List<Classblock> getStudentClassblocksOnDate(int idstudent, Date date) {
		List<Classblock> list = new ArrayList<Classblock>();
		
		try {
			String sql = "SELECT classblock.* FROM classblock, term, " + database + ".group, group_has_student " +
					"WHERE classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') " +
					"AND term_idterm=idterm AND '" + dateFormat.format(date) + "' BETWEEN term.start AND term.end " +
					"AND classblock.group_idgroup=" + database + ".group.idgroup " +
					"AND " + database + ".group.idgroup=group_has_student.group_idgroup " +
					"AND group_has_student.student_idstudent=" + idstudent + " " +
					"ORDER BY classblock.start;";
			
			ResultSet result = statement.executeQuery(sql);
			
			while(result.next()) {
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("teacher_idteacher"), result.getInt("term_idterm"),result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				list.add(block);
			}
			
		} catch(SQLException e) {
			e.printStackTrace();
		}
		
		return list;
	}
	
	public void testConnectionPool() {
		try {
			
			InitialContext ctx = new InitialContext();
			DataSource ds = (DataSource)ctx.lookup("java:comp/env/jdbc/TestDB");
			
			Connection conn = null;
	        Statement stmt = null;
	        
	        conn = ds.getConnection();
	        
	        stmt = conn.createStatement();
            stmt.execute("SELECT 1;");
            
            stmt.close();
            stmt = null;

            conn.close();
            conn = null;
	        
		} catch (NamingException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
}
