/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
import java.util.LinkedHashMap;
import java.util.Vector;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Image;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.ColumnText;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPageEventHelper;
import com.itextpdf.text.pdf.PdfWriter;



/**
 * Esta clase permite generar informes en formato PDF. Los informes se escriben
 * en un ByteArrayOutputStream. Cada tipo de informe se genera en función
 * de un id de grupo y de una fecha de inicio y otra de final, creando un informe
 * entre ambas fechas y el grupo dado.
 * 
 * @author vcarceler <vcarceler@iespuigcastellar.xeill.net>
 */

public class CreatePDF  {


	private ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
	private DatabaseStorage database = AttendancemanagerApplication.storage;
	private DateFormat df = AttendancemanagerApplication.df;
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	
	private Font h1Font = new Font(Font.FontFamily.HELVETICA, 14);
	private Font h2Font = new Font(Font.FontFamily.HELVETICA, 10);
	private Font tableTitleFont = new Font(Font.FontFamily.HELVETICA, 8);
	private Font cellFont = new Font(Font.FontFamily.HELVETICA, 8);
	private Font monoFont = new Font(Font.FontFamily.COURIER, 8);
	private Font footerFont = new Font(Font.FontFamily.HELVETICA, 8);
	private Float grayCell = 0.8F;
	
	private final int MAX_ROWS_WITHOUT_FLUSH = 200;
	
	public class HeaderFooterHelper extends PdfPageEventHelper {
		protected Phrase header;
		protected Phrase footer;
		
		public HeaderFooterHelper(Phrase header) {
			this.header = header;
		}
		
		public void onEndPage(PdfWriter writer, Document document) {
			PdfContentByte cb = writer.getDirectContent();
			
//			if (document.getPageNumber() > 1)
//				ColumnText.showTextAligned(cb, Element.ALIGN_LEFT, header, document.leftMargin(), document.top(), 0);
			
			footer = new Phrase(locale.getString("CREATEPDF_CAPTION_PAGENUMBER") + document.getPageNumber(), footerFont);
			ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, footer, (document.right() - document.left()) / 2
					+ document.leftMargin(), document.bottom() - footerFont.getSize() - 10, 0);
		}
	}
	
	private class DocumentHelper extends Document {
		public DocumentHelper(OutputStream os, Rectangle pagesize, String title, int idgroup, Date dfecha1, Date dfecha2) throws DocumentException, MalformedURLException, IOException {
			super(pagesize, 50, 50, 50, 50);
			PdfWriter writer = PdfWriter.getInstance(this, os);
			writer.setPageEvent(new HeaderFooterHelper( null ));
			
			open();
			addCreator("AttendanceManager");
			addTitle(title);
			addCreationDate();
			
			Image img1 = Image.getInstance(rb.getString("report_logo_file"));
			img1.scalePercent(100*72/Integer.parseInt(rb.getString("dpi_logo_file")));
			add(img1);
			
			Chunk c1 = new Chunk(title + "\n", h1Font);
			c1.setUnderline(0.2f, -2f);
			add(new Paragraph(c1));
			
			Chunk c2 = new Chunk(locale.getString("CREATEPDF_CAPTION_GROUP") + database.getGroupName(idgroup), h2Font);
			add(new Paragraph(c2));
			
			Teacher teacher = database.getGroupTeacher(idgroup);
			Chunk c3 = new Chunk(locale.getString("CREATEPDF_CAPTION_TUTOR") + teacher.getFullname(), h2Font);
			add(new Paragraph(c3));
			
			Chunk c4 = new Chunk(locale.getString("CREATEPDF_CAPTION_TERM") + df.format(dfecha1) + " - " + df.format(dfecha2), h2Font);
			add(new Paragraph(c4));
		}
	}
	
	@SuppressWarnings("serial")
	private class ListHelper<T> extends ArrayList<T> {
		private Date date = null;
		public Date getDate() {
			return date;
		}

		public void setDate(Date date) {
			this.date = date;
		}
	}

	/**
	 * Generates a group report.
	 * 
	 * @param idgroup
	 * @param dfecha1
	 * @param dfecha2
	 * @return
	 * @throws SQLException
	 * @throws DocumentException
	 * @throws MalformedURLException
	 * @throws IOException
	 * @throws ParseException
	 */
	public ByteArrayOutputStream createReport1(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, DocumentException, MalformedURLException, IOException, ParseException{
		
		Logger.log("CreatePDF.createReport1(idgroup: " + idgroup + ", dfecha1: " + dfecha1 + ", dfecha2: " +dfecha2 + ")");

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DocumentHelper document = new DocumentHelper(baos, PageSize.A4, locale.getString("CREATEPDF_REPORT1_TITLE"), idgroup, dfecha1, dfecha2);
		
		
		PdfPTable table = new PdfPTable(7);
		table.setSpacingBefore(20);
		table.setWidthPercentage(100);
		float widths[] = {30.3f, 30.3f, 30.3f, 20f, 20f, 20f, 20f};
		table.setWidths(widths);
		table.setHeaderRows(1);
		
		String columns[] = {locale.getString("CREATEPDF_TABLECOLUMN_SURNAME1"), locale.getString("CREATEPDF_TABLECOLUMN_SURNAME2"), locale.getString("CREATEPDF_TABLECOLUMN_NAME"), locale.getString("CREATEPDF_TABLECOLUMN_MISS"), locale.getString("CREATEPDF_TABLECOLUMN_EXCUSED"), locale.getString("CREATEPDF_TABLECOLUMN_DELAY"), locale.getString("CREATEPDF_TABLECOLUMN_EXPULSION")};
		for(String column: columns) {
			PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
			cell.setGrayFill(grayCell);
			table.addCell(cell);
		}
		

		ResultSet result = database.studentReport1(idgroup, dfecha1, dfecha2);
	
		int i=0;
		while (result.next()) {
			i++;
			String nombre = result.getString("name"); 			if (nombre.equals("null")) nombre="";
			String apellido1 = result.getString("surname1");	if (apellido1.equals("null")) apellido1="";
			String apellido2 = result.getString("surname2");	if (apellido2.equals("null")) apellido2="";
			int miss= result.getInt("misscount");
			int excused= result.getInt("excusedcount");
			int delay= result.getInt("delaycount");
			int expulsion= result.getInt("expulsioncount");
			
			PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(apellido1, monoFont)));
			PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(apellido2, monoFont)));
			PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(nombre, monoFont)));
			PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(""+miss, monoFont)));
			PdfPCell aux5= new PdfPCell(new Paragraph(new Chunk(""+excused, monoFont)));
			PdfPCell aux6= new PdfPCell(new Paragraph(new Chunk(""+delay, monoFont)));
			PdfPCell aux7= new PdfPCell(new Paragraph(new Chunk(""+expulsion, monoFont)));
			
			table.addCell(aux1);
			table.addCell(aux2);
			table.addCell(aux3);
			
			aux4.setHorizontalAlignment(Element.ALIGN_RIGHT); table.addCell(aux4);
			aux5.setHorizontalAlignment(Element.ALIGN_RIGHT); table.addCell(aux5);
			aux6.setHorizontalAlignment(Element.ALIGN_RIGHT); table.addCell(aux6);
			aux7.setHorizontalAlignment(Element.ALIGN_RIGHT); table.addCell(aux7);
			
			// On a big table flush memory every MAX_ROWS
			if (i % MAX_ROWS_WITHOUT_FLUSH == 0) {
				document.add(table);
				table.deleteBodyRows();
				table.setSkipFirstHeader(true);
				table.setSpacingBefore(0);
			}
		}
		
		document.add(table);
		document.close();
		
		return baos;
	}

	/**
	 * 
	 * @param idgroup
	 * @param dateA
	 * @param dateB
	 * @return
	 * @throws SQLException
	 * @throws DocumentException
	 * @throws MalformedURLException
	 * @throws IOException
	 * @throws ParseException
	 */
	public ByteArrayOutputStream createReport2(int idgroup, Date dateA, Date dateB) throws SQLException, DocumentException, MalformedURLException, IOException, ParseException{
		
		Logger.log("CreatePDF.createReport2(idgroup: " + idgroup + ", dfecha1: " + dateA + ", dfecha2: " + dateB + ")" );
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DocumentHelper document = new DocumentHelper(baos, PageSize.A4, locale.getString("CREATEPDF_REPORT2_TITLE"), idgroup, dateA, dateB);
		
		
		PdfPTable table = new PdfPTable(6);
		table.setSpacingBefore(20);
		table.setWidthPercentage(100);
		float widths[] = {55.5f, 55.5f, 15f, 15f, 15f, 15f};
		table.setWidths(widths);
		table.setHeaderRows(1);
		
		String columns[] = {locale.getString("CREATEPDF_TABLECOLUMN_STUDENT"), locale.getString("CREATEPDF_TABLECOLUMN_SUBJECT"), locale.getString("CREATEPDF_TABLECOLUMN_MISS"), locale.getString("CREATEPDF_TABLECOLUMN_EXCUSED"), locale.getString("CREATEPDF_TABLECOLUMN_DELAY"), locale.getString("CREATEPDF_TABLECOLUMN_EXPULSION")};
		for(String column: columns) {
			PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
			cell.setGrayFill(grayCell);
			table.addCell(cell);
		}
		
		// Get the collection of students in group and report miss count
		Collection<Student> students = database.getStudentsCollection(idgroup);
		Iterator<Student> it = students.iterator();
		while(it.hasNext()) {
			Student student = it.next();
			System.out.println(student);
			
			ResultSet result = database.reportStudentMissCount(student.getID(), dateA, dateB);
			int rowcount=0;
			Vector<PdfPCell> missCells = new Vector<PdfPCell>();
			while(result.next()) {
				rowcount++;
				System.out.println("Asignatura: " + result.getString("subject_name") + " Faltas: " + result.getInt("miss") + " Justificadas: " + result.getInt("excused")+ " Retrasos: " + result.getInt("delay")+ " Expulsiones: " + result.getInt("expulsion"));
				
				PdfPCell fullname =  new PdfPCell(new Paragraph(new Chunk(result.getString("subject_name"), cellFont)));
				PdfPCell miss = new PdfPCell(new Paragraph(new Chunk("" + result.getInt("miss"), cellFont)));
				PdfPCell excused = new PdfPCell(new Paragraph(new Chunk("" + result.getInt("excused"), cellFont)));
				PdfPCell delay = new PdfPCell(new Paragraph(new Chunk("" + result.getInt("delay"), cellFont)));
				PdfPCell expulsion = new PdfPCell(new Paragraph(new Chunk("" + result.getInt("expulsion"), cellFont)));
				
				miss.setHorizontalAlignment(Element.ALIGN_RIGHT);
				excused.setHorizontalAlignment(Element.ALIGN_RIGHT);
				delay.setHorizontalAlignment(Element.ALIGN_RIGHT);
				expulsion.setHorizontalAlignment(Element.ALIGN_RIGHT);
				
				missCells.add( fullname );
				missCells.add( miss );
				missCells.add( excused );
				missCells.add( delay );
				missCells.add( expulsion );
			}
			
			System.out.println("Registros: " + rowcount);
			if(rowcount>0) {
				PdfPCell studentCell = new PdfPCell( new Paragraph(new Chunk(student.getFullName(), cellFont)) );
				studentCell.setRowspan(rowcount);
				table.addCell(studentCell);
				
				for(PdfPCell cell : missCells) {
					table.addCell(cell);
				}
			}
			
			result.close();
		}
		
		document.add(table);
		document.close();

		return baos;
	}
	
	public ByteArrayOutputStream createReport3(int idgroup, Date dateA, Date dateB) throws DocumentException, MalformedURLException, IOException, ParseException {
		
//		dateA = new Date(2010-1900, 7, 9);
//		Date dateB = new Date(2010-1900, 7, 13);
//		System.out.println("dateA: " + dateA + "\tdateB: " + dateB);
		
		Logger.log("CreatePDF.createReport3(idgroup: " + idgroup + ", dateA: " + dateA + ", dateB: " + dateB + ")" );
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DocumentHelper document = new DocumentHelper(baos, PageSize.A4.rotate(), locale.getString("CREATEPDF_REPORT3_TITLE"), idgroup, dateA, dateB);
		
		/*
		 * 1. Iterar por los días de dateA a dateB recogiendo los classblocks
		 *    del grupo-term en una colección de colecciones (una colección
		 *    que contiene las diferentes colecciones (una por día) con los
		 *    classblocks del día).
		 * 2. Los Classblocks de un día están agrupados en una entidad ListHelper
		 * 	  que permite especificar la fecha a la que corresponden.
		 */
		
		// Colección de classblocks, una colección por día.
		List<ListHelper<Classblock>> classblocks = new ArrayList<ListHelper<Classblock>>();
		int nclassblocks=0; // Número total de classblocks
		
		Calendar c = Calendar.getInstance();
		c.setTime(dateA);
		Calendar cmax = Calendar.getInstance();
		cmax.setTime(dateB);
		cmax.add(Calendar.DAY_OF_YEAR, 1);
		while(c.before(cmax)) {

			ListHelper<Classblock> dayclassblocks = new ListHelper<Classblock>();
			dayclassblocks.addAll( AttendancemanagerApplication.storage.getClassblocksOnDate(idgroup, c.getTime()) );

			if(!dayclassblocks.isEmpty()) {
				dayclassblocks.setDate(c.getTime());
				classblocks.add(dayclassblocks);
			}
			nclassblocks += dayclassblocks.size();
			
			c.add(Calendar.DAY_OF_YEAR, 1);
		}
		
		// Tabla con una columna para los nombres de alumno + nclassblocks columnas
		PdfPTable table = new PdfPTable(1 + nclassblocks);
		table.setSpacingBefore(20);
		table.setHorizontalAlignment(Element.ALIGN_LEFT);
		table.setHeaderRows(2);
		table.setWidthPercentage(100f);
		int widths[] = new int[1 + nclassblocks]; //{10,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
		widths[0] = 10;
		for(int i=1; i<widths.length; i++) widths[i] = 1;
		table.setWidths(widths);
		
		// Generamos la cabecera con dos filas: días y horas
		PdfPCell emptyCell = new PdfPCell(new Paragraph(new Chunk(""))); emptyCell.setGrayFill(grayCell); table.addCell(emptyCell); // Celda vacía
		for(ListHelper<Classblock> dayclassblocks: classblocks) {
			PdfPCell day = new PdfPCell(new Paragraph(new Chunk(dayclassblocks.get(0).getDayName() + " - " + AttendancemanagerApplication.df.format(dayclassblocks.getDate()), tableTitleFont)));
			day.setGrayFill(grayCell);
			day.setHorizontalAlignment(Element.ALIGN_CENTER);
			day.setColspan(dayclassblocks.size());
			table.addCell(day);
		}
		
		PdfPCell studentsCell = new PdfPCell(new Paragraph(new Chunk(locale.getString("CREATEPDF_TABLECOLUMN_STUDENT"), tableTitleFont)));
		studentsCell.setVerticalAlignment(Element.ALIGN_BOTTOM);
		studentsCell.setGrayFill(grayCell);
		table.addCell(studentsCell);
		for(List<Classblock> dayclassblocks: classblocks) {
			for(Classblock cb: dayclassblocks) {
				PdfPCell hour = new PdfPCell(new Paragraph(new Chunk(DatabaseStorage.timeFormat.format(cb.getStart()) + " - " + DatabaseStorage.timeFormat.format(cb.getEnd()), tableTitleFont)));
				hour.setGrayFill(grayCell);
				hour.setRotation(90);
				table.addCell(hour);
			}
		}
		
		// Generamos las filas correspondientes a los alumnos
		List<Student> students = AttendancemanagerApplication.storage.getStudentsCollection(idgroup);
		for(Student student: students) {
			PdfPCell student_name = new PdfPCell(new Paragraph(new Chunk(student.getFullName(), cellFont)));
			table.addCell(student_name);
			
			for(ListHelper<Classblock> dayclassblocks: classblocks) {
				for(Classblock cb: dayclassblocks) {
					String code = "";
					Date date = dayclassblocks.getDate();
					
					if (AttendancemanagerApplication.storage.existsMiss(Miss.MISS, date, cb.getID(), student.getID())) code += locale.getString("CREATEPDF_CODE_MISS");
					if (AttendancemanagerApplication.storage.existsMiss(Miss.EXCUSED_MISS, date, cb.getID(), student.getID())) code += locale.getString("CREATEPDF_CODE_EXCUSED");
					if (AttendancemanagerApplication.storage.existsMiss(Miss.DELAY, date, cb.getID(), student.getID())) code += locale.getString("CREATEPDF_CODE_DELAY");
					if (AttendancemanagerApplication.storage.existsMiss(Miss.EXPULSION, date, cb.getID(), student.getID())) code += locale.getString("CREATEPDF_CODE_EXPULSION");
					
					PdfPCell misscell = new PdfPCell(new Paragraph(new Chunk(code, cellFont)));
					table.addCell(misscell);
				}
			}
		}
		
		
		document.add(table);
		document.close();
		
		return baos;
	}
	
	/**
	 * Llistat de faltes de grup.
	 * 
	 * @param idgroup
	 * @param dfecha1
	 * @param dfecha2
	 * @return
	 * @throws SQLException
	 * @throws DocumentException
	 * @throws MalformedURLException
	 * @throws IOException
	 * @throws ParseException
	 */
	public ByteArrayOutputStream createReport4(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, DocumentException, MalformedURLException, IOException, ParseException{
		
		Logger.log("CreatePDF.createReport4(idgroup: " + idgroup + ", dfecha1: " + dfecha1 + ", dfecha2: " +dfecha2 + ")");

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DocumentHelper document = new DocumentHelper(baos, PageSize.A4, locale.getString("CREATEPDF_REPORT4_TITLE"), idgroup, dfecha1, dfecha2);
		
		
		PdfPTable table = new PdfPTable(4);
		table.setSpacingBefore(20);
		table.setWidthPercentage(100);
		float widths[] = {60f, 60f, 30f, 21f};
		table.setWidths(widths);
		table.setHeaderRows(1);
		
		String columns[] = {locale.getString("CREATEPDF_TABLECOLUMN_STUDENT"), locale.getString("CREATEPDF_TABLECOLUMN_CLASSBLOCK"), locale.getString("CREATEPDF_TABLECOLUMN_SUBJECT"), locale.getString("CREATEPDF_TABLECOLUMN_DESCRIPTION")};
		for(String column: columns) {
			PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
			cell.setGrayFill(grayCell);
			table.addCell(cell);
		}
		

		ResultSet result = database.studentReport4(idgroup, dfecha1, dfecha2);
	
		int i=0;
		while (result.next()) {
			i++;
			
			Student student = new Student(result.getString("surname1"), result.getString("surname2"), result.getString("name"));
			String fullname = student.getFullName();
			
			Date date = result.getDate("date");
			int day_of_week = result.getInt("day_of_week");
			String time = result.getString("start") + "-" + result.getString("end");
			String classblock = df.format(date) + " " + time + " " + Classblock.getDayName(day_of_week);
			
			String subject = result.getString("subject");
			
			String description = Miss.getDescription(result.getInt("type"));
			
			PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(fullname, monoFont)));
			PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(classblock, monoFont)));
			PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(subject, monoFont)));
			PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(description, monoFont)));
			
			table.addCell(aux1);
			table.addCell(aux2);
			table.addCell(aux3);
			table.addCell(aux4);
			
			// On a big table flush memory every MAX_ROWS
			if (i % MAX_ROWS_WITHOUT_FLUSH == 0) {
				document.add(table);
				table.deleteBodyRows();
				table.setSkipFirstHeader(true);
				table.setSpacingBefore(0);
			}
		}
		
		document.add(table);
		document.close();
		
		return baos;
	}
	
	/**
	 * Genera una parrilla de faltas para el grupo y las fechas indicadas, se incluyen
	 * las faltas de los alumnos del grupo que han sido cometidas en otros grupos.
	 * 
	 * @param idgroup
	 * @param dateA
	 * @param dateB
	 * @return
	 * @throws DocumentException
	 * @throws MalformedURLException
	 * @throws IOException
	 * @throws ParseException
	 */
	public ByteArrayOutputStream createReport5(int idgroup, Date dateA, Date dateB) throws DocumentException, MalformedURLException, IOException, ParseException {
		
		Logger.log("CreatePDF.createReport5(idgroup: " + idgroup + ", dateA: " + dateA + ", dateB: " + dateB + ")" );
		
		/*
		 * Comenzamos creando una estructura de datos en memoria que nos
		 * permita generar la tabla con comodidad.
		 * 
		 * La estructura de datos es:
		 * 
		 *  LinkedHashMap que indexando por Date guarda:
		 *    LinkedHashMap que indexando por String guarda:
		 *      LinkedHashMap que indexando por Student guarda:
		 *        Lista de Classblocks.
		 *        
		 */
		LinkedHashMap<Date, LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>>> grid;
		grid = new LinkedHashMap<Date, LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>>>();
		
		Collection<Student> students = database.studentsOfGroup(idgroup);
		
		int nclassblocks = 0;
		
		/*
		 * Cargamos la estructura con los datos
		 */
		Calendar c = Calendar.getInstance();
		c.setTime(dateA);
		Calendar cmax = Calendar.getInstance();
		cmax.setTime(dateB);
		cmax.add(Calendar.DAY_OF_YEAR, 1);
		while(c.before(cmax)) {
			Date date = c.getTime();
			
			LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>> day = new LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>>();
			
			// Para cada estudiante del grupo, averiguamos sus clases
			for(Student student : students) {
				List<Classblock> classblocks = database.getStudentClassblocksOnDate(student.getID(), date);
				// y las insertamos en el día si no existe o actualizamos los datos
				for(Classblock classblock : classblocks) {
					String tr = DatabaseStorage.timeFormat.format(classblock.getStart()) + " - " + DatabaseStorage.timeFormat.format(classblock.getEnd());
					LinkedHashMap<Student, ArrayList<Miss>> column = null;
					
					if(day.containsKey(tr)) {
						column = day.get(tr);
					} else {
						column = new LinkedHashMap<Student, ArrayList<Miss>>();
					}
					
					column.put(student, database.getStudentMisses(student.getID(), classblock.getID(), date));
					day.put(tr, column);
				}
			}
			
			if(!day.isEmpty()) {
				grid.put(date, day);
				nclassblocks += day.size();
			}
			c.add(Calendar.DAY_OF_YEAR, 1);
		}
		
		System.out.println("nclassblocks: " + nclassblocks);
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DocumentHelper document = new DocumentHelper(baos, PageSize.A4.rotate(), locale.getString("CREATEPDF_REPORT5_TITLE"), idgroup, dateA, dateB);
		
		// Tabla con una columna para los nombres de alumno + nclassblocks columnas
		PdfPTable table = new PdfPTable(1 + nclassblocks);
		table.setSpacingBefore(20);
		table.setHorizontalAlignment(Element.ALIGN_LEFT);
		table.setHeaderRows(2);
		table.setWidthPercentage(100f);
		int widths[] = new int[1 + nclassblocks]; //{10,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
		widths[0] = 10;
		for(int i=1; i<widths.length; i++) widths[i] = 1;
		table.setWidths(widths);
		
		// Generamos la cabecera con dos filas: días y horas
		PdfPCell emptyCell = new PdfPCell(new Paragraph(new Chunk(""))); emptyCell.setGrayFill(grayCell); table.addCell(emptyCell); // Celda vacía
		for(Date date : grid.keySet()) {
			PdfPCell day = new PdfPCell(new Paragraph(new Chunk( AttendancemanagerApplication.df.format(date), tableTitleFont)));
			day.setGrayFill(grayCell);
			day.setHorizontalAlignment(Element.ALIGN_CENTER);
			day.setColspan(grid.get(date).size());
			table.addCell(day);
		}
		
		PdfPCell studentsCell = new PdfPCell(new Paragraph(new Chunk(locale.getString("CREATEPDF_TABLECOLUMN_STUDENT"), tableTitleFont)));
		studentsCell.setVerticalAlignment(Element.ALIGN_BOTTOM);
		studentsCell.setGrayFill(grayCell);
		table.addCell(studentsCell);
		for(Date date : grid.keySet()) {
			LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>> day = grid.get(date);
			
			for(String tr : day.keySet()) {
				PdfPCell hour = new PdfPCell(new Paragraph(new Chunk(tr, tableTitleFont)));
				hour.setGrayFill(grayCell);
				hour.setRotation(90);
				table.addCell(hour);
			}
		}

		
		// Generamos las filas correspondientes a los alumnos
		for(Student student: students) {
			PdfPCell student_name = new PdfPCell(new Paragraph(new Chunk(student.getFullName(), cellFont)));
			table.addCell(student_name);
			
			for(Date date : grid.keySet()) {
				LinkedHashMap<String, LinkedHashMap<Student, ArrayList<Miss>>> day = grid.get(date);
				
				for(String tr : day.keySet()) {
					ArrayList<Miss> misses = day.get(tr).get(student);
					String code = "";
					
					if(misses != null){
						for(Miss miss : misses) {
							if (miss.getType() == Miss.MISS) code += locale.getString("CREATEPDF_CODE_MISS");
							if (miss.getType() == Miss.EXCUSED_MISS) code += locale.getString("CREATEPDF_CODE_EXCUSED");
							if (miss.getType() == Miss.DELAY) code += locale.getString("CREATEPDF_CODE_DELAY");
							if (miss.getType() == Miss.EXPULSION) code += locale.getString("CREATEPDF_CODE_EXPULSION");
						}
					}
					
					PdfPCell misscell = new PdfPCell(new Paragraph(new Chunk(code, cellFont)));
					table.addCell(misscell);
				}
				
			}
			

		}
		
		
		document.add(table);
		document.close();
		
		return baos;
	}
	
	public ByteArrayOutputStream createReport6(Student student, Date start, Date end) {
		Logger.log("CreatePDF.createReport6(student: " + student + ", dfecha1: " + start + ", dfecha2: " + end + ")");
		
		List<Miss> missesMiss = database.getStudentMisses(student.getID(), start, end, Miss.MISS);
		List<Miss> missesExcused = database.getStudentMisses(student.getID(), start, end, Miss.EXCUSED_MISS);
		List<Miss> missesDelay = database.getStudentMisses(student.getID(), start, end, Miss.DELAY);
		List<Miss> missesExpulsion = database.getStudentMisses(student.getID(), start, end, Miss.EXPULSION);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Document document = new Document(PageSize.A4, 50, 50, 50, 50);
		
		PdfWriter writer;
		try {
			writer = PdfWriter.getInstance(document, baos);
			writer.setPageEvent(new HeaderFooterHelper( null ));
			
			document.open();
			document.addCreator("AttendanceManager");
			document.addTitle(locale.getString("CREATEPDF_REPORT6_TITLE"));
			document.addCreationDate();
			
			Image img1 = Image.getInstance(rb.getString("report_logo_file"));
			img1.scalePercent(100*72/Integer.parseInt(rb.getString("dpi_logo_file")));
			document.add(img1);
			
			Chunk c1 = new Chunk(locale.getString("CREATEPDF_REPORT6_TITLE") + ": " + student.getFullName() + "\n", h1Font);
			c1.setUnderline(0.2f, -2f);
			document.add(new Paragraph(c1));
			
			Chunk c2 = new Chunk(locale.getString("CREATEPDF_CAPTION_TERM") + df.format(start) + " - " + df.format(end), h2Font);
			document.add(new Paragraph(c2));
			
			Chunk c3 = new Chunk(locale.getString("CREATEPDF_MISS_COUNT") + missesMiss.size(), h2Font);
			document.add(new Paragraph(c3));
			
			Chunk c4 = new Chunk(locale.getString("CREATEPDF_EXCUSED_COUNT") + missesExcused.size(), h2Font);
			document.add(new Paragraph(c4));
			
			Chunk c5 = new Chunk(locale.getString("CREATEPDF_DELAY_COUNT") + missesDelay.size(), h2Font);
			document.add(new Paragraph(c5));
			
			Chunk c6 = new Chunk(locale.getString("CREATEPDF_EXPULSION_COUNT") + missesExpulsion.size(), h2Font);
			document.add(new Paragraph(c6));

			PdfPTable table = new PdfPTable(3);
			table.setSpacingBefore(20);
			table.setWidthPercentage(100);
			float widths[] = {/*60f,*/ 60f, 30f, 21f};
			table.setWidths(widths);
			table.setHeaderRows(1);
			
			String columns[] = {/*locale.getString("CREATEPDF_TABLECOLUMN_STUDENT"),*/ locale.getString("CREATEPDF_TABLECOLUMN_CLASSBLOCK"), locale.getString("CREATEPDF_TABLECOLUMN_SUBJECT"), locale.getString("CREATEPDF_TABLECOLUMN_DESCRIPTION")};
			for(String column: columns) {
				PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
				cell.setGrayFill(grayCell);
				table.addCell(cell);
			}
			
			List<Miss> misses = new ArrayList<Miss>();
			misses.addAll(missesMiss);
			misses.addAll(missesExcused);
			misses.addAll(missesDelay);
			misses.addAll(missesExpulsion);
			
			int i=0;
			for(Miss miss : misses) {
				i++;
				
				Classblock cb = database.getClassblock(miss.getIDClassblock());
				String time = cb.getStart() + "-" + cb.getEnd();
				String classblock = df.format(miss.getDate()) + " " + time + " " + Classblock.getDayName(cb.getDay_of_week());
				
				String subject = database.getSubject(cb.getIDSubject()).getName();
				
				String description = Miss.getDescription(miss.getType());
				
				//PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(student.getFullName(), monoFont)));
				PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(classblock, monoFont)));
				PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(subject, monoFont)));
				PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(description, monoFont)));
				
				//table.addCell(aux1);
				table.addCell(aux2);
				table.addCell(aux3);
				table.addCell(aux4);
				
				// On a big table flush memory every MAX_ROWS
				if (i % MAX_ROWS_WITHOUT_FLUSH == 0) {
					document.add(table);
					table.deleteBodyRows();
					table.setSkipFirstHeader(true);
					table.setSpacingBefore(0);
				}
			}
			
			document.add(table);
			
		} catch (Exception e) {
			e.printStackTrace();
		}	
		
		document.close();
		
		return baos;
	}
	
	// TODO smsList y mailList comparten el 99% del código, lo suyo es 
	// refactorizar estos métodos.
	public ByteArrayOutputStream smsList(Collection<NotificationReceipt> receipts) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Document document = new Document(PageSize.A4, 50, 50, 50, 50);
		
		PdfWriter writer;
		try {
			writer = PdfWriter.getInstance(document, baos);
			writer.setPageEvent(new HeaderFooterHelper( null ));
			
			document.open();
			document.addCreator("AttendanceManager");
			document.addTitle(locale.getString("CREATEPDF_SMSLIST_TITLE"));
			document.addCreationDate();
			
			Image img1 = Image.getInstance(rb.getString("report_logo_file"));
			img1.scalePercent(100*72/Integer.parseInt(rb.getString("dpi_logo_file")));
			document.add(img1);
			
			Chunk c1 = new Chunk(locale.getString("CREATEPDF_SMSLIST_TITLE") + "\n", h1Font);
			c1.setUnderline(0.2f, -2f);
			document.add(new Paragraph(c1));
			
			Chunk c2 = new Chunk(locale.getString("CREATEPDF_SMSLIST_TRIED") + receipts.size(), h2Font);
			document.add(new Paragraph(c2));
			
			int delivered=0;
			int notdelivered=0;
			for(NotificationReceipt receipt : receipts) {
				if(receipt.status) delivered++; else notdelivered++;
			}
			
			Chunk c3 = new Chunk(locale.getString("CREATEPDF_SMSLIST_DELIVERED") + delivered, h2Font);
			document.add(new Paragraph(c3));
			
			Chunk c4 = new Chunk(locale.getString("CREATEPDF_SMSLIST_NOTDELIVERED") + notdelivered, h2Font);
			document.add(new Paragraph(c4));
			
			PdfPTable table = new PdfPTable(5);
			table.setSpacingBefore(20);
			table.setWidthPercentage(100);
			float widths[] = {15f, 15f, 20f, 15f, 106f};
			table.setWidths(widths);
			table.setHeaderRows(1);
			
			String columns[] = {locale.getString("CREATEPDF_TABLECOLUMN_DATE"), locale.getString("CREATEPDF_TABLECOLUMN_TIME"), locale.getString("CREATEPDF_TABLECOLUMN_SMSPHONE"), locale.getString("CREATEPDF_TABLECOLUMN_STATUS"), locale.getString("CREATEPDF_TABLECOLUMN_TEXT")};
			for(String column: columns) {
				PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
				cell.setGrayFill(grayCell);
				table.addCell(cell);
			}
			
			int i=0;
			for(NotificationReceipt receipt : receipts) {
				PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(df.format(receipt.date), monoFont)));
				PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(DatabaseStorage.timeFormat.format(receipt.date), monoFont)));
				PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(receipt.address, monoFont)));
				PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(receipt.status?locale.getString("CREATEPDF_SMSLIST_YES"):locale.getString("CREATEPDF_SMSLIST_NO"), monoFont)));
				PdfPCell aux5= new PdfPCell(new Paragraph(new Chunk(receipt.text.replace("\n", " "), monoFont)));
				
				table.addCell(aux1);
				table.addCell(aux2);
				table.addCell(aux3);
				table.addCell(aux4);
				table.addCell(aux5);
				
				// On a big table flush memory every MAX_ROWS
				if (i % MAX_ROWS_WITHOUT_FLUSH == 0) {
					document.add(table);
					table.deleteBodyRows();
					table.setSkipFirstHeader(true);
					table.setSpacingBefore(0);
				}
				
				i++;
			}
			
			document.add(table);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	
		document.close();
		
		return baos;
	}
	
	public ByteArrayOutputStream mailList(Collection<NotificationReceipt> receipts) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Document document = new Document(PageSize.A4, 50, 50, 50, 50);
		
		PdfWriter writer;
		try {
			writer = PdfWriter.getInstance(document, baos);
			writer.setPageEvent(new HeaderFooterHelper( null ));
			
			document.open();
			document.addCreator("AttendanceManager");
			document.addTitle(locale.getString("CREATEPDF_SMSLIST_TITLE"));
			document.addCreationDate();
			
			Image img1 = Image.getInstance(rb.getString("report_logo_file"));
			img1.scalePercent(100*72/Integer.parseInt(rb.getString("dpi_logo_file")));
			document.add(img1);
			
			Chunk c1 = new Chunk(locale.getString("CREATEPDF_SMSLIST_TITLE") + "\n", h1Font);
			c1.setUnderline(0.2f, -2f);
			document.add(new Paragraph(c1));
			
			Chunk c2 = new Chunk(locale.getString("CREATEPDF_SMSLIST_TRIED") + receipts.size(), h2Font);
			document.add(new Paragraph(c2));
			
			int delivered=0;
			int notdelivered=0;
			for(NotificationReceipt receipt : receipts) {
				if(receipt.status) delivered++; else notdelivered++;
			}
			
			Chunk c3 = new Chunk(locale.getString("CREATEPDF_SMSLIST_DELIVERED") + delivered, h2Font);
			document.add(new Paragraph(c3));
			
			Chunk c4 = new Chunk(locale.getString("CREATEPDF_SMSLIST_NOTDELIVERED") + notdelivered, h2Font);
			document.add(new Paragraph(c4));
			
			PdfPTable table = new PdfPTable(5);
			table.setSpacingBefore(20);
			table.setWidthPercentage(100);
			float widths[] = {15f, 15f, 40f, 15f, 86f};
			table.setWidths(widths);
			table.setHeaderRows(1);
			
			String columns[] = {locale.getString("CREATEPDF_TABLECOLUMN_DATE"), locale.getString("CREATEPDF_TABLECOLUMN_TIME"), locale.getString("CREATEPDF_TABLECOLUMN_MAIL"), locale.getString("CREATEPDF_TABLECOLUMN_STATUS"), locale.getString("CREATEPDF_TABLECOLUMN_TEXT")};
			for(String column: columns) {
				PdfPCell cell = new PdfPCell(new Paragraph(new Chunk(column, tableTitleFont)));
				cell.setGrayFill(grayCell);
				table.addCell(cell);
			}
			
			int i=0;
			for(NotificationReceipt receipt : receipts) {
				PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(df.format(receipt.date), monoFont)));
				PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(DatabaseStorage.timeFormat.format(receipt.date), monoFont)));
				PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(receipt.address, monoFont)));
				PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(receipt.status?locale.getString("CREATEPDF_SMSLIST_YES"):locale.getString("CREATEPDF_SMSLIST_NO"), monoFont)));
				PdfPCell aux5= new PdfPCell(new Paragraph(new Chunk(receipt.text.replace("\n", " "), monoFont)));
				
				table.addCell(aux1);
				table.addCell(aux2);
				table.addCell(aux3);
				table.addCell(aux4);
				table.addCell(aux5);
				
				// On a big table flush memory every MAX_ROWS
				if (i % MAX_ROWS_WITHOUT_FLUSH == 0) {
					document.add(table);
					table.deleteBodyRows();
					table.setSkipFirstHeader(true);
					table.setSpacingBefore(0);
				}
				
				i++;
			}
			
			document.add(table);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	
		document.close();
		
		return baos;
	}
	
}