/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.amtools;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.SMS;

/**
 * @author aalejandro Alex Amigo <aalejando@iespuigcastellar.xeill.net>
 * @author Victor Carceler <vcarceler@iespuigcastellar.xeill.net>
 * 
 * Versión modificada a partir del código de Alex Amigo en SMSGroupNotifier.
 * 
 * Esta versión permite notificar faltas:
 * 
 *  a) En un periodo de tiempo especificado por dos fechas
 *  b) En un día contando faltas posteriores a la hora indicada
 * 
 
 usage: SMSGroupNotifier -date <date> [-date_end <arg>] -group <group>
       -message <message> [-singular_message <arg>] [-test] [-time <arg>]
       -type <type>
 -date <date>              Data inicial pel recompte de faltes. Si només
                           s'especifica la data inicial es comptaran les
                           faltes d'aquest día. Format: 17/05/11
 -date_end <arg>           Data final pel recompte de faltes. Si no
                           s'especifica només es tenen en compte les
                           faltes de la data indicada
 -group <group>            ID del grup
 -message <message>        Missatge a enviar als alumnes amb faltes. Si no
                           s'especifica el missatge en singular, aquest
                           missatge es farà servir per tots el cassos. Si
                           s'especifica un missatge en singular aquest
                           missatge només es fará servir pels alumnes que
                           tinguin més d'una falta. Es possible emprar els
                           seguents camps: %n -> nom, %a -> primer cognom,
                           %b -> segon cognom, %c -> nom amb els dos
                           cognoms concatenats, %d -> data, %e -> data
                           final, %f -> nombre de faltes
 -singular_message <arg>   Missatge a enviar en el cas d'una falta
 -test                     Realitza una simul·lació pero no envía cap
                           missatge
 -time <arg>               Només es comptaran les faltes a partir de
                           l'hora indicada, incompatible amb la opció
                           'date_end'. Format: 12:16:42
 -type <type>              Tipus de falta a comptar: 1 -> Falta, 2 ->
                           Justificada, 3 -> Retard, 4 -> Expulsió
                           
 * 
 * El programa permite notificar por SMS las faltas de un grupo. Los parámetros
 * del programa son:
 * 
 * 	-test -> Opcional, si está presente no se envían los SMS pero se indican
 *           los mensajes que se enviarían.
 *           
 *  -group <n> -> Identificador del grupo
 *  
 *  -type <n>  -> Tipo de falta a notificar
 *  
 *  -date <fecha> -> Fecha del día en el que se buscarán faltas a notificar.
 *                   Si además se especifica el parámetro date_end entonces
 *                   se trata de la fecha inicial del periodo en el que se
 *                   recontarán las faltas.
 *                   
 *  -date_end <fecha> -> Fecha final para el periodo de recuento de faltas
 *  
 *  -message <cadena> -> Cadena de formato para los mensajes a enviar
 *  
 *  -singular_message <cadena> -> Cadena de formato para los mensajes en los
 *                                que sólo se notifica una falta (Opcional)
 *                                
 *  -time <hora> -> Cuando se especifica una hora sólo se notifican faltas
 *                  posteriores a la hora indicada. Este parámetro es
 *                  incompatible con -date_end. (Opcional)
 *
 */
@SuppressWarnings("static-access")
public class SMSGroupNotifierv2 {

	private DatabaseStorage db = new DatabaseStorage();
	private AttendancemanagerApplication app = new AttendancemanagerApplication();
	private ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
	private DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));
	private SMS sms = new SMS();
	private Options options = new Options();
	//private String[] missType = {"NOT USED", "misscount", "excusedcount", "delaycount", "expulsioncount"};
	private ResultSet resultset;
	private int missCount;
	private String smsphone;

	private int test = 0;	// test, se utiliza para comprobar si el primer campo es --test, o no, 0 no se ha introducido.
	private int group;		//
	private int type;
	private Date date;
	private Date date_end = null;
	private String message;
	private String singular_message = null;
	private Time time = null;

	
	

	public static void main(String[] args) {
		
		SMSGroupNotifierv2 app = new SMSGroupNotifierv2();
		
		app.checkParameters(args);
		try {
			app.doit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
     * Este método comprueba si los argumentos introducidos por el usuario
     * son correctos. Comprueba la cantidad de argumentos y el tipo de cada uno
     * de ellos.
     *      
     * @param    args    Un <code>Array</code> de <code>String</code> con los argumentos
     * introducidos por el usuario.
     */
	public void checkParameters(String[] args){
		
		options.addOption("test", false, app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_TEST"));
		options.addOption("singular_message", true, app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_SINGULARMESSAGE"));
		options.addOption("time", true, app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_TIME") + " " + new Time(System.currentTimeMillis()));
		options.addOption("date_end", true, app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_DATEEND"));
		
		Option group = OptionBuilder.withArgName("group")
						.hasArg()
						.withDescription(app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_GROUP"))
						.create("group");
		group.setRequired(true);
		options.addOption(group);
		
		Option type = OptionBuilder.withArgName("type")
						.hasArg()
						.withDescription(app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_TYPE"))
						.create("type");
		type.setRequired(true);
		options.addOption(type);
		
		Option date = OptionBuilder.withArgName("date")
						.hasArg()
						.withDescription(app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_DATE") + " " + df.format(new Date()) )
						.create("date");
		date.setRequired(true);
		options.addOption(date);

		Option message = OptionBuilder.withArgName("message")
						.hasArg()
						.withDescription(app.locale.getString("SMSGROUPNOTIFIER_OPTIONS_MESSAGE"))
						.create("message");
		message.setRequired(true);
		options.addOption(message);
		
		
		CommandLineParser parser = new GnuParser();
		CommandLine cmd = null;
		
		try {
			cmd = parser.parse(options, args);
		} catch (Exception e) {
			// Faltan opciones requeridas o los argumentos no tienen el formato adecuado
			// Listado de grupos + help + sintaxis de uso
			printGroups();
			printUsage();
			System.exit(2);
		}
			
		if(cmd.hasOption("test")) test = 1;
		
		try {
			this.group = Integer.parseInt( cmd.getOptionValue("group") );
		} catch (Exception e) {
			printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_IDGROUP"));
			System.exit(3);
		}
		
		try {
			this.type = Integer.parseInt( cmd.getOptionValue("type") );
			if(this.type < 1 || this.type > 4 ) throw new Exception();
		} catch (Exception e) {
			printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_MISS"));
			System.exit(4);
		}
		
		try {
			this.date = df.parse( cmd.getOptionValue("date") );
		} catch (ParseException e) {
			printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_DATE") + df.format(new Date()));
			System.exit(5);
		}
		
		try {
			if(cmd.hasOption("date_end")) this.date_end = df.parse( cmd.getOptionValue("date_end") ); else date_end = this.date;
		} catch (Exception e) {
			printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_DATE") + df.format(new Date()));
			System.exit(6);
		}
		
		this.message = cmd.getOptionValue("message");
		if(!checkSmsText(this.message)) {
			printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_SMS"));
			System.exit(7);
		}
		
		if(cmd.hasOption("singular_message")) {
			this.singular_message = cmd.getOptionValue("singular_message");
			if(!checkSmsText(this.singular_message)) {
				printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_SMS"));
				System.exit(8);
			}
		}
		
		if(cmd.hasOption("time")) {
			if(cmd.hasOption("date_end")) {
				printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_TIMEDATEEND"));
				System.exit(8);
			}
			try {
				this.time = Time.valueOf( cmd.getOptionValue("time") );
			} catch (Exception e) {
				printUsage(app.locale.getString("SMSGROUPNOTIFIER_ERROR_TIME") + " " + new Time(System.currentTimeMillis()));
				System.exit(8);
			}
		}
	}
	

	public void doit() throws SQLException {
		
		// Si me pasan una hora, sólo busco faltas del día 'date' posteriores a la hora indicada
		if(time!=null) {
			resultset = db.studentReport5(this.group, this.type, this.date, this.time);
		} else {
			// Faltas entre dos fechas: si date_end no se ha especificado tiene el mismo valor
			// que date. Así que solo se cuentan faltas de un día.
			resultset = db.studentReport5(this.group, this.type, this.date, this.date_end);
		}
		
		while(resultset.next()) {
			//int c = resultset.getInt(missType[type]);
			int c = resultset.getInt("misscount");
			String msg;
			
			if(singular_message != null && c == 1){
				msg = replace(singular_message);
			} else {
				msg = replace(message);
			}
			
			if(missCount > 0) {
				if(!checkLengthSmsText(msg)) System.err.println("SMS No enviado. Longitud excesiva. MSG=" + msg);
				if(smsphone==null) System.err.println("SMS No enviado. Teléfono nulo. MSG=" + msg);
				if( test!=1 && sms.sendSMS(smsphone, msg) ) {
					System.out.println("SMS enviado. Número: " + smsphone + " MSG=" + msg);
				} else {
					if(test!=1) {
						System.err.println("SMS No enviado. Error en el envío. Número: " + smsphone + " MSG=" + msg);
					} else {
						System.err.println("Modo TEST. Número: " + smsphone + " MSG=" + msg);
					}
				}
			}
		}
	
	}
	
	
	/**
     * Este método comprueba la cadena del SMS. Dada una cadena
     * introducida por el usuario, comprueba si es correcta. Comprueba
     * el tipo de parámetros opciones del mensaje. 
     *
     * @param    text    Una <code>String</code> con el texto a enviar.
     * 
     * @return cierto si el texto a enviar es correcto.
     */
	public boolean checkSmsText(String text){
		String aux = "";
		
		for (int i = 0; i < text.length(); i++){
			if (aux.compareTo("%") == 0){
				if (text.charAt(i) == '%' || (text.charAt(i) != 'n' && text.charAt(i) != 'a' && text.charAt(i) != 'b' && text.charAt(i) != 'c' && text.charAt(i) != 'd' && text.charAt(i) != 'e' && text.charAt(i) != 'f')){
					return false;
				}
				else{
					aux += text.charAt(i);
				}
			}

			else if (text.charAt(i) == '%') aux = "%";
		}
		return true;
	}
	
	/**
     * Este método comprueba la longitud de la cadena del SMS. Dada una cadena
     * introducida por el usuario, comprueba si no supera una longitud determinada
	 * por el programa. Este valor depende el número de mensajes concatenados 
	 * permitidos para enviar.
	 *
     * @param    text    Una <code>String</code> con el texto a enviar.
     * 
     * @return cierto si la longitud del texto a enviar es correcta.
     */
	 public boolean checkLengthSmsText(String text){
		 int sms_n = Integer.parseInt(rb.getString("sms_max_number"));
		 if (sms_n == 1){
			 if (text.length() > 70){
				 Logger.log(app.locale.getString("SMSGROUPNOTIFIER_LOG_LONGSMS") + text);
				 System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_LONGSMS" + text));
				 return false;
			 }
		 }
		
		 else if (text.length() > (63 * sms_n)){
			 Logger.log(app.locale.getString("SMSGROUPNOTIFIER_LOG_LONGSMS") + text);
			 System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_LONGSMS") + text);
			 return false;
		 }
		 
		 return true;
	}
	
	

	@Override
	public String toString() {
		return "SMSGroupNotifierv2 [db=" + db + ", app=" + app + ", rb=" + rb
				+ ", df=" + df + ", test=" + test + ", group=" + group
				+ ", type=" + type + ", date=" + date + ", date_end="
				+ date_end + ", message=" + message + ", singular_message="
				+ singular_message + ", time=" + time + ", sms=" + sms + "]";
	}
	
	
	
	public void printGroups() {
		Collection<Group> grupos = db.allGroups();
		System.out.println(app.locale.getString("SMSGROUPNOTIFIER_GROUPINFO"));
		for (Group gr : grupos) {
			System.out.println(gr.getID() + " -- " + gr.getName() + " -- " + gr.getComment());
		}
	}
	
	public void printUsage() {
		printUsage(null);
	}
	
	public void printUsage(String err) {
		if (err != null) System.err.println("\n" + err + "\n");
		
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("SMSGroupNotifier", options, true);
		
		System.out.println();
		printHelp();
	}
	
	public void printHelp() {
		System.err.println(app.locale.getString("SMSGROUPNOTIFIER_HELP_INSTRUCTIONS") + 
				app.locale.getString("SMSGROUPNOTIFIER_HELP_OPTIONAL") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_TEST") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_LISTGROUP") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_DATE") + df.format(new Date()) + "\n\n" +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSA") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSB") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERS") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSN") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSA") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSB") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSC") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSD") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSF") +
				app.locale.getString("SMSGROUPNOTIFIER_HELP_EXAMPLE"));
	}
	
	/*
	 * Cambia los códigos de control de la cadena por sus valores
	 */
	public String replace(String s) {
		String result = null;
		
		try {
			//missCount = resultset.getInt(missType[type]);
			missCount = resultset.getInt("misscount");
			smsphone = resultset.getString("student.smsphone");
			result = s.replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d", df.format(date)).replace("%e", df.format(date_end)).replace("%f", "" + missCount);
		} catch (SQLException e) {
			e.printStackTrace();
			System.exit(10);
		}
		
		return result;
	}
}
